#include <iostream>
#include <algorithm>
#include <vector>
#include <map>
#include <queue>
using namespace std;

#define endl '\n'
#define int long long

#pragma GCC optimize("Ofast")

int timer = 0;

int mm = 17;

struct Node
{
	vector<pair<int, int>> to;
	pair<int, int> parent = { -1, -1 };

	vector<pair<int, int>> up = vector<pair<int, int>>(mm);

	int tin = -1;
	int tout = -1;

	bool used = false;
};

vector<vector<Node>> graph;

void dfs(int vx, int vy)
{
	graph[vx][vy].tin = ++timer;

	for (int i = 0; i < graph[vx][vy].to.size(); i++)
	{
		int ux = graph[vx][vy].to[i].first;
		int uy = graph[vx][vy].to[i].second;

		dfs(ux, uy);
	}

	graph[vx][vy].tout = ++timer;
}

bool is_parent(pair<int, int> x, pair<int, int> y)
{
	int x1 = x.first;
	int y1 = x.second;

	int x2 = y.first;
	int y2 = y.second;

	if (graph[x1][y1].tin <= graph[x2][y2].tin && graph[x1][y1].tout >= graph[x2][y2].tout)
	{
		return true;
	}
	else
	{
		return false;
	}
}

pair<int, int> lca(pair<int, int> p1, pair<int, int> p2)
{
	if (is_parent(p1, p2))
	{
		return p1;
	}
	else if (is_parent(p2, p1))
	{
		return p2;
	}

	for (int i = mm - 1; i >= 0; i--)
	{
		pair<int, int> np = graph[p1.first][p1.second].up[i];

		if (!is_parent(np, p2))
		{
			p1 = np;
		}
	}

	return graph[p1.first][p1.second].up[0];
}

void task()
{
	int n;
	cin >> n;

	graph.resize(n + 1, vector<Node>(n + 1));

	vector<int> route(n, 1);
	pair<int, int> cur = { 0,0 };

	for (int i = 0; i < n; i++)
	{
		pair<int, int> ne = cur;
		ne.first++;
		ne.second += route[i];

		if (!graph[ne.first][ne.second].used)
		{
			graph[ne.first][ne.second].used = true;
			graph[cur.first][cur.second].to.push_back({ ne.first, ne.second });
		}

		cur = ne;

		graph[cur.first][cur.second].parent = { cur.first - 1, cur.second - route[i] };
	}

	for (int i = 0; i < n; i++)
	{
		int f;
		cin >> f;
		f--;

		route[f] = 0;

		cur = { 0,0 };

		for (int i = 0; i < n; i++)
		{
			pair<int, int> ne = cur;
			ne.first++;
			ne.second += route[i];

			if (!graph[ne.first][ne.second].used)
			{
				graph[ne.first][ne.second].used = true;
				graph[cur.first][cur.second].to.push_back({ ne.first, ne.second });
			}

			cur = ne;

			graph[cur.first][cur.second].parent = { cur.first - 1, cur.second - route[i] };
		}
	}

	for (int i = 0; i < n + 1; i++)
	{
		for (int j = 0; j < n + 1; j++)
		{
			if (graph[i][j].parent != make_pair((int)-1, (int)-1))
			{
				graph[i][j].up[0] = graph[i][j].parent;
			}
		}
	}

	for (int k = 1; k < mm; k++)
	{
		for (int i = 0; i < n + 1; i++)
		{
			for (int j = 0; j < n + 1; j++)
			{
				if (graph[i][j].parent != make_pair((int)-1, (int)-1))
				{
					int px = graph[i][j].up[k - 1].first;
					int py = graph[i][j].up[k - 1].second;
					graph[i][j].up[k] = graph[px][py].up[k - 1];
				}
			}
		}
	}

	dfs(0, 0);

	int q;
	cin >> q;

	while (q--)
	{
		int a, b, c, d;
		cin >> a >> b >> c >> d;

		pair<int, int> l = lca({ a,b }, { c,d });

		cout << l.first << ' ' << l.second << endl;
	}
}

signed main()
{
	ios_base::sync_with_stdio(false);
	cin.tie(nullptr);
	cout.tie(nullptr);

	task();

	return 0;
}
