#include <iostream>
#include <vector>
using namespace std;

const int maxn = 5e6;
vector<int> g[maxn];
int zero = maxn - 1;
int lft[maxn];
int deep[maxn];
pair<int, int> A[maxn];

int get_index(int x, int y) {
    return x * (x + 1) / 2 + y;
}

int get_index(pair<int, int> p) {
    auto [x, y] = p;
    return get_index(x, y);
}


struct segtree {
    static const int size = 1024 * 1024 * 8;

    int tree[size * 2 - 1];

    segtree() {
        fill(tree, tree + size * 2 - 1, zero);
    }

    void set(int i, int u, int x = 0, int lx = 0, int rx = size) {
        if (rx - lx == 1) {
            tree[x] = u;
            return;
        }

        if (i < (lx + rx) / 2) set(i, u, x * 2 + 1, lx, (lx + rx) / 2);
        else set(i, u, x * 2 + 2, (lx + rx) / 2, rx);

        if (deep[tree[x * 2 + 1]] < deep[tree[x * 2 + 2]]) {
            tree[x] = tree[x * 2 + 1];
        }
        else {
            tree[x] = tree[x * 2 + 2];
        }
    }

    int get_min(int l, int r, int x = 0, int lx = 0, int rx = size) {
        if (l <= lx && rx <= r) return tree[x];
        if (l >= rx || lx >= r) return zero;

        int u = get_min(l, r, x * 2 + 1, lx, (lx + rx) / 2);
        int v = get_min(l, r, x * 2 + 2, (lx + rx) / 2, rx);

        if (deep[u] > deep[v]) return v;
        else return u;
    }
};

segtree st;
int sz = 0;

void dfs(int u, int d = 0) {
    deep[u] = d;

    for (int v : g[u]) {
        lft[u] = min(lft[u], sz);
        st.tree[st.size + sz - 1] = u;
        sz++;
        st.tree[st.size + sz - 1] = v;
        lft[v] = min(lft[v], sz);
        sz++;
        dfs(v, d + 1);
    }
}


int lca(int u, int v) {
    int l = min(lft[u], lft[v]);
    int r = max(lft[u], lft[v]);
    return st.get_min(l, r + 1);
}

signed main() {
    ios::sync_with_stdio(false);
    cin.tie(nullptr);
    for (int i = 0; i < maxn; i++) {
        g[i].reserve(2);
    }
    fill(lft, lft + maxn, INT32_MAX);
    st = segtree();
    deep[zero] = INT32_MAX;
    int n; cin >> n;

    vector<pair<int, int>> q;
    q.emplace_back(0, 0);
    for (int i = 1; i <= n; i++) {
        g[get_index(q.back())].push_back(get_index(i, i));
        q.emplace_back(i, i);
    }

    for (int i = 0; i <= n; i++) {
        for (int j = 0; j <= i; j++) {
            A[get_index(i, j)] = {i, j};
        }
    }

    for (int i = 1; i <= n; i++) {
        int v; cin >> v;
        for (int j = v; j <= n; j++) {
            q[j].second--;
        }
        for (int j = v; j <= n; j++) {
            g[get_index(q[j - 1])].push_back(get_index(q[j]));
        }
    }

    dfs(0);
    for (int i = st.size - 2; i >= 0; i--) {
        if (deep[st.tree[i * 2 + 1]] < deep[st.tree[i * 2 + 2]]) {
            st.tree[i] = st.tree[i * 2 + 1];
        }
        else {
            st.tree[i] = st.tree[i * 2 + 2];
        }
    }

    int t; cin >> t;
    while (t--) {
        int r1, c1; cin >> r1 >> c1;
        int r2, c2; cin >> r2 >> c2;
        int u = get_index(r1, c1);
        int v = get_index(r2, c2);

        auto [r, c] = A[lca(u, v)];
        cout << r << " " << c << "\n";
    }
    return 0;
}