#include <iostream>
#include <vector>
#include <functional>
#include <cassert>

#define int int64_t

using namespace std;

using graph_t = vector<vector<int>>;

const int ROOT = 0;

const int64_t INF = (1LL << 57);

struct HLD {
    vector<int> euler_tour;
    vector<int> tree;
    vector<int> pushed;
    graph_t childrens;
    int sz, tree_sz;
    int timer;
    vector<int> tins, touts;

    HLD(const graph_t &childrens) {
        int n = childrens.size();
        sz = n;
        tree_sz = 4 * n + 5;
        this->childrens = childrens;
        tree.resize(tree_sz);
        pushed.resize(tree_sz);
        tins.resize(sz);
        touts.resize(sz);
        timer = 0;
        build_tour(ROOT);
    }

    void build_tour(int v) {
        tins[v] = timer++;
        euler_tour.push_back(v);
        for (int u : childrens[v]) {
            build_tour(u);
        }
        touts[v] = timer;
    }

    int get(int v) {
        return tree_get(0, 0, sz, tins[v], touts[v]);
    }

    void add(int v, int c) {
        tree_add(0, 0, sz, tins[v], tins[v + 1], c);
    }

    void push(int v, int l, int r) {
        int old = pushed[v];
        tree[v] += old * 1ll * (r - l);
        pushed[v] = 0;
        if (l + 1 == r) {
            return;
        }
        pushed[2 * v + 1] += old;
        pushed[2 * v + 2] += old;
    }

    int tree_get(int v, int l, int r, int ql, int qr) {
        if (l >= qr || ql >= r) {
            return 0;
        }
        push(v, l, r);
        if (ql <= l && r <= qr) {
            return tree[v];
        }
        int m = (l + r) / 2;
        return tree_get(2 * v + 1, l, m, ql, qr) + tree_get(2 * v + 2, m, r, ql, qr);
    }

    void tree_add(int v, int l, int r, int ql, int qr, int val) {
        push(v, l, r);
        if (l >= qr || ql >= r) {
            return;
        }
        if (ql <= l && r <= qr) {
            pushed[v] += val;
            return;
        }
        int m = (l + r) / 2;
        tree_add(2 * v + 1, l, m, ql, qr, val);
        tree_add(2 * v + 2, m, r, ql, qr, val);
        push(2 * v + 1, l, m); push(2 * v + 2, m, r);
        tree[v] = tree[2 * v + 1] + tree[2 * v + 2];
    }
};

void solve() {
    int n; cin >> n;
    vector<int> parents(n, -1), costs(n);
    graph_t childrens(n);
    for (int i = 1; i < n; i++) {
        cin >> parents[i];
        parents[i]--;
        childrens[parents[i]].push_back(i);
    }
    for (int i = 0; i < n; i++) {
        cin >> costs[i];
    }
    vector<int> ls(n), rs(n);
    for (int i = 0; i < n; i++) {
        cin >> ls[i] >> rs[i];
    }
    vector<int> available_vertices;
    vector<int> nums(n);
    vector<pair<int64_t, int>> best_vertex(n, {INF, -1});
    int64_t ans = 0;
    HLD hld(childrens);
    function<void(int)> dfs = [&](int v) {
        best_vertex[v] = {costs[v], v};
        for (int u : childrens[v]) {
            dfs(u);
            best_vertex[v] = min(best_vertex[v], best_vertex[u]);
        }
        int64_t cur = hld.get(v);
        int u = best_vertex[v].second;
        if (cur < ls[v]) {
            nums[u] += ls[v] - cur;
            ans += (ls[v] - cur) * 1ll * costs[u];
            hld.add(u, ls[v] - cur);
        }
    };
    dfs(ROOT);
    cout << ans << "\n";
    for (int i = 0; i < n; i++) {
        cout << nums[i] << ' ';
    }
    cout << "\n";
}

signed main() {
    int t; cin >> t;
    while (t--) {
        solve();
    }
}
