#include <iostream>
#include <vector>
#include <functional>
#include <set>
#include <algorithm>
#include <cassert>
#include <ext/pb_ds/tree_policy.hpp>
#include <ext/pb_ds/assoc_container.hpp>
using namespace __gnu_pbds;

using namespace std;
#define int long long

typedef tree<int, null_type, less<int>, rb_tree_tag, tree_order_statistics_node_update> ordered_set;

const int inf = 1e18;

struct Tree {
  vector<ordered_set> tree;
  int s;

  Tree(int n) {
    s = 1;
    while (s < n) s *= 2;
    tree.resize(2 * s + 1);
  }

  void add(int i, int val, int x, int lx, int rx) {
    tree[x].insert(val);
    if (lx + 1 == rx) return;
    int m = (lx + rx) / 2;
    if (i < m) {
      add(i, val, 2 * x + 1, lx, m);
    } else {
      add(i, val, 2 * x + 2, m, rx);
    }
  }

  int get_less(int l, int r, int val, int x, int lx, int rx) {
    if (lx >= r || rx <= l) return 0;
    if (lx >= l && rx <= r) return tree[x].order_of_key(val + 1);
    int m = (lx + rx) / 2;
    return get_less(l, r, val, 2 * x + 1, lx, m) + get_less(l, r, val, 2 * x + 2, m, rx);
  }

  int get_pos(int cnt_ops, int need_times, int x, int lx, int rx) {
    if (lx + 1 == rx) {
      return lx;
    }
    int left_side = tree[2 * x + 1].order_of_key(cnt_ops );
//    cout << "do: " << lx << ' ' << rx << ' ' << cnt_ops << ' ' << need_times << ' ' << left_side << endl;
    int m = (lx + rx) / 2;
    if (left_side >= need_times) {
      return get_pos(cnt_ops, need_times, 2 * x + 1, lx, m);
    } else {
      return get_pos(cnt_ops, need_times - left_side, 2 * x + 2, m, rx);
    }
  }

  int get_first(int r, int val1, int val2, int x, int lx, int rx) {
    if (lx >= r) return inf;
    if (lx + 1 == rx) return *tree[x].begin();
    int cnt1 = tree[2 * x + 1].order_of_key(val1 + 1);
    int cnt2 = tree[2 * x + 1].order_of_key(val2 + 1);
    if (cnt1 == cnt2) return inf;
    int m = (lx + rx) / 2;
    return min(get_first(r, val1, val2, 2 * x + 1, lx, m), get_first(r, val1, val2, 2 * x + 2, m, rx));
  }
};

struct TreeMx {
  vector<set<int>> tree;
  int s;

  TreeMx(int n) {
    s = 1;
    while (s < n) s *= 2;
    tree.resize(2 * s + 1);
  }

  void add(int i, int val, int x, int lx, int rx) {
    tree[x].insert(val);
    if (lx + 1 == rx) return;
    int m = (lx + rx) / 2;
    if (i < m) {
      add(i, val, 2 * x + 1, lx, m);
    } else {
      add(i, val, 2 * x + 2, m, rx);
    }
  }

  int get_first(int n1, int n2, int x, int lx, int rx) {
    if (lx + 1 == rx) {
      return lx;
    }
    int m = (lx + rx) / 2;
    auto it = tree[2 * x + 1].upper_bound(min(n1, n2));
    if (it != tree[2 * x + 1].end() && *it <= max(n1, n2)) {
      return get_first(n1, n2, 2 * x + 1, lx, m);
    } else {
      return get_first(n1, n2, 2 * x + 2, m, rx);
    }
  }
};

void solve() {
  int n;
  cin >> n;
  vector<int> ar(n);
  for (auto &x : ar) {
    cin >> x;
    --x;
  }
  int q;
  cin >> q;

  Tree st(n);
  for (int i = 0; i < n; i++) {
    st.add(i, ar[i], 0, 0, st.s);
  }

  TreeMx st1(n);
  for (int i = 0; i < n; i++) {
    st1.add(ar[i], i, 0, 0, st1.s);
  }

  auto get_op = [&] (int x, int y) -> int {
//    cout << x << ' ' << y << endl;
    if (x == 0) {
      return -1;
    } else {
      int cnt_ops = x + y;
//      cout << "cnt_ops:" << cnt_ops << ' ' << x << endl;
      return st.get_pos(cnt_ops, x, 0, 0, st.s);
    }
  };

  vector<array<int, 4>> queries(q);
  for (auto &[a, b, c, d] : queries) {
    cin >> a >> b >> c >> d;
    if (make_pair(a, b) == make_pair(c, d)) {
      cout << a << ' ' << b << '\n';
      continue;
    }
    a -= b;
    c -= d;
//    cout << a << ' ' << b << ' ' << c << ' ' << d << endl;
    int n1 = get_op(a, b), n2 = get_op(c, d);
//    cout << n1 << ' ' << n2 << endl;
    int kok = st1.get_first(n1, n2, 0, 0, st.s);
//    int kok = st.get_first(min(a + b, c + d), n1, n2, 0, 0, st.s);
    int x, y;
//    cout << "kok: " << kok << '\n';
    if (kok == -1) {
      x = 0, y = 0;
    } else {
      x = st.get_less(0, kok, n1, 0, 0, st.s);
      y = kok - x;
    }
    x += y;
    cout << x << ' ' << y << '\n';
  }
}

signed main() {
  ios_base::sync_with_stdio(0);
  cin.tie(0);
  solve();
  return 0;
}
