#include <iostream>
#include <algorithm>
#include <cmath>
#include <string>
#include <vector>
#include <random>
#include <chrono>
#include <set>
#include <map>
#include <unordered_map>
#include <iomanip>
#include <stack>
#include <deque>
#include <queue>

#define all(a) (a).begin(), (a).end()

using ll = long long;
using ld = long double;
using pll = std::pair<ll, ll>;
using str = std::string;

const ll Max = 1e17, Min = -1e17;
const ld Pi = std::acos(-1);
const ll Mod = 1234567891;
const ld eps = 1e-12;

struct SegTree {
	std::vector<ll> tree;
	ll size = 1;

	void build(ll n) {
		while (size < n) {
			size *= 2;
		}
		tree.assign(size * 2 - 1, 0);
	}

	ll get(ll l_find, ll r_find, ll l, ll r, ll i) {
		if (l >= r_find || r <= l_find) return 0;
		if (l >= l_find && r <= r_find) return tree[i];
		ll mid = l + (r - l) / 2;
		return get(l_find, r_find, l, mid, i * 2 + 1) + get(l_find, r_find, mid, r, i * 2 + 2);
	}

	void change(ll pos, ll val, ll l, ll r, ll i) {
		if (l > pos || r <= pos) return;
		if (r - l == 1) {
			tree[i] = val;
			return;
		}
		ll mid = l + (r - l) / 2;
		change(pos, val, l, mid, i * 2 + 1);
		change(pos, val, mid, r, i * 2 + 2);
		tree[i] = tree[i * 2 + 1] + tree[i * 2 + 2];
	}
};

void solve() {
	ll n;
	std::cin >> n;
	std::vector<ll> list(n);
	for (int i = 0; i < n; ++i) {
		std::cin >> list[i];
	}
	SegTree tree;
	tree.build(n + 1);
	std::vector<ll> pref(n, 0), suf(n, 0);
	for (int i = 0; i < n; ++i) {
		pref[i] = tree.get(list[i], n + 1, 0, tree.size, 0);
		tree.change(list[i], 1, 0, tree.size, 0);
	}
	tree.build(n + 1);
	for (int i = n - 1; i >= 0; --i) {
		suf[i] = tree.get(0, list[i], 0, tree.size, 0);
		tree.change(list[i], 1, 0, tree.size, 0);
	}
	ll ans = 0;
	for (ll i = 0; i < n; ++i) {
		for (ll j = i + 1; j < n; ++j) {
			ll max = std::max({ std::abs(list[i] - list[j]), j - i + pref[i], j - i + suf[j], j - i });
			ans += max;
		}
	}
	std::cout << ans << '\n';
}

int main() {
	std::ios::sync_with_stdio(0);
	std::cin.tie(0);
	std::cout.tie(0);
	std::cout << std::fixed << std::setprecision(100);
	ll t = 1;
	// std::cin >> t;
	while (t--) {
		solve();
	}
}
