#include <bits/stdc++.h>

typedef long long ll;
typedef long double ld;
using namespace std;

const ll INF = 2e9;
const int MAXR = 1001;

struct SegmentTree {
    int tree_size;
    vector <int> tree, push;

    SegmentTree(int n) {
        tree_size = 1;
        while (tree_size < n)
            tree_size <<= 1;

        tree.assign(2 * tree_size, INF);
        push.resize(2 * tree_size);
    }

    void pushv(int v) {
        if (push[v] == 0) return;

        push[2 * v] += push[v];
        tree[2 * v] += push[v];

        push[2 * v + 1] += push[v];
        tree[2 * v + 1] += push[v];

        push[v] = 0;
    }

    void update(int v, int tl, int tr, int pos, int value)  {;
        if (tr - tl == 1) {
            tree[v] = value;
            return;
        }

        pushv(v);

        int mid = (tl + tr) / 2;
        if (pos < mid) {
            update(2 * v, tl, mid, pos, value);
        } else {
            update(2 * v  + 1, mid, tr, pos, value);
        }

        tree[v] = min(tree[2 * v], tree[2 * v + 1]);
    }

    void update(int pos, int value) {
        update(1, 0, tree_size, pos, value);
    }

    void add(int v, int tl, int tr, int l, int r, int dt)  {
        if (tl >= r || tr <= l)
            return;

        if (tl >= l && tr <= r) {
            tree[v] += dt;
            push[v] += dt;
            return;
        }

        pushv(v);

        int mid = (tl + tr) / 2;
        add(2 * v, tl, mid, l, r, dt);
        add(2 * v + 1, mid, tr, l, r, dt);

        tree[v] = min(tree[2 * v], tree[2 * v + 1]);
    }

    void add(int l, int r, int dt) {
        add(1, 0, tree_size, l, r, dt);
    }

    int query(int v, int tl, int tr, int l, int r) {
        if (tl >= r || tr <= l)
            return INF;

        if (tl >= l && tr <= r)
            return tree[v];

        pushv(v);

        int mid = (tl + tr) / 2;
        int left = query(2 * v, tl, mid, l, r);
        int right = query(2 * v + 1, mid, tr, l, r);
        return min(left, right);
    }

    int query(int l, int r) {
        return query(1, 0, tree_size, l, r);
    }
};

void dfs1(vector <vector <int>> &graph, vector <int> &sz, int u) {
    for  (int v: graph[u]) {
        dfs1(graph, sz, v);
        sz[u] += sz[v];
    }
}

void dfs2(vector <vector <int>> &graph, vector <int> &heavy, vector  <int> &sz, vector <int> &order, int u, int &t) {
    order[u] = t++;

    int x = -1;
    for (int v: graph[u]) {
        if (x == -1 || sz[v] > sz[x]) {
            x = sz[v];
        }
    }

    if (x != -1) {
        heavy[x] = heavy[u] != -1 ? heavy[u] : u;
        dfs2(graph, heavy, sz, order, x, t);
    }

    for (int v: graph[u]) {
        if (v == x) continue;
        dfs2(graph, heavy, sz, order, v, t);
    }
}

int get_capacity(SegmentTree &tree, vector <int> &p, vector <int> &heavy, vector <int> &cap, vector <int> &order, int v, int until) {
    int result = INF;

    if (heavy[until] != -1) {
        result = tree.query(order[until], order[until] + 1);
    } else {
        result = cap[until];
    }

    while (order[v] > order[until]) {
        if (heavy[v] != -1) {
            result = min(result, tree.query(max(order[heavy[v]], order[until] + 1), order[v] + 1));
            v = heavy[v];
        } else {
            result = min(result, cap[v]);
            v = p[v];
        }
    }

    return result;
}

void add(SegmentTree &tree, vector <int> &p, vector <int> &heavy, vector <int> &cap, vector <int> &order, int v, int until, int dt) {
    if (heavy[until] > 0) {
        tree.add(order[until], order[until] + 1, dt);
    } else {
        cap[until] += dt;
    }

    while (order[v] > order[until]) {
        if (heavy[v] != -1) {
            tree.add(max(order[heavy[v]], order[until] + 1), order[v] + 1, dt);
            v = heavy[v];
        } else {
            cap[v] += dt;
            v = p[v];
        }
    }
}

bool dfs3(
          SegmentTree &tree,
          vector <vector <int>> &graph,
          vector <int> &p, vector <int> &heavy, vector <int> &order,
          vector <int> &cap, vector <ll> &l, vector <ll> &r, vector <ll> &cost,
          vector <ll> &ans1, vector <ll> &ans2,
          multiset <pair <int, int>> &rs,
          int u
) {
    ll ls = 0;
    multiset <pair <int, int>> s;
    s.emplace(cost[u], u);

    for (int v: graph[u]) {
        dfs3(tree, graph, p, heavy, order, cap, l, r, cost, ans1, ans2, s, v);
        ls += l[v];
        ans1[u] += ans1[v];
    }

    if (ls > r[u]) {
        return true;
    }

    while (ls < l[u]) {
        if (s.empty()) return true;
        auto [c, v] = *s.begin();
        s.erase(s.begin());

        ll v_capacity = get_capacity(tree, p, heavy, cap, order, v, u);
        ll common = min(v_capacity, l[u] - ls);

        ls += common;
        add(tree, p, heavy, cap, order, v, u, -common);

        ans1[u] += c * common;
        ans2[v] += common;

        if (v_capacity > common) {
            s.emplace(c, v);
        }
    }

    l[u] = max(l[u], ls);

    if (heavy[u] != -1) {
        tree.update(order[u], r[u] - l[u]);
    } else {
        cap[u] = r[u] - l[u];
    }

    if (rs.size() < s.size())
        rs.swap(s);

    for (auto p: s)
        rs.insert(p);
    s.clear();

    return false;
}

void solve() {
    int n;
    cin >> n;

    vector <int> p(n);
    vector <vector <int>> graph(n);
    for (int i = 1; i < n; i++) {
        cin >> p[i]; p[i]--;
        graph[p[i]].emplace_back(i);
    }

    vector <ll> cost(n);
    for (ll &x: cost) cin >> x;

    vector <ll> l(n), r(n);
    for (int i = 0; i < n; i++)
        cin >> l[i] >> r[i];

    int t = 0;
    vector <int> sz(n, 1), order(n), heavy(n, -1);
    dfs1(graph, sz, 0);
    dfs2(graph, heavy, sz, order, 0, t);

    vector <int> cap(n, INF);
    SegmentTree tree(n);

    vector <ll> ans1(n), ans2(n);
    multiset <pair <int, int>> s;
    if (dfs3(tree, graph, p, heavy, order, cap, l, r, cost, ans1, ans2, s, 0)) {
        cout << "-1\n";
        return;
    }

    for (int i = 0; i < n; i++) {
        if (l[i] > r[i]) {
            cout << "-1\n";
            return;
        }
    }

    cout << ans1[0] << '\n';
    for (ll &x: ans2) cout << x << ' ';
    cout << '\n';
}

signed main() {
    cin.tie(NULL);
    ios_base::sync_with_stdio(false);

    #ifdef local
    freopen("input.txt", "r", stdin);
    #endif // local

    int t;
    cin >> t;

    while (t--)
    solve();
}
