#include <iostream>
#include <vector>
#include <algorithm>
#include <cmath>
#include <set>
#include <map>
#include <queue>
#include <string>
#include <cassert>

using namespace std;

const int MAXN = 1e5 + 7;
const long long inf = 1e18 + 7;

int p[MAXN];
vector <int> g[MAXN];
int c[MAXN];
int l[MAXN];
int r[MAXN];

int w[MAXN];

void build1(int v) {
	w[v] = 1;
	for (auto to : g[v]) {
		build1(to);
		w[v] += w[to];
	}
	if (g[v].size()) {
		int id = 0;
		for (int i = 1; i < g[v].size(); i++)
			if (w[g[v][i]] > w[g[v][id]])
				id = i;
		swap(g[v][0], g[v][id]);
	}
}

int sta[MAXN];
int dep[MAXN];
int h[MAXN];

vector <int> et;

void build2(int v, int head) {
	sta[v] = (int)et.size();
	h[v] = head;
	et.push_back(r[v]);
	for (auto to : g[v])
		dep[to] = dep[v] + 1;
	if (g[v].size()) {
		build2(g[v][0], v);
		for (int i = 1; i < g[v].size(); i++)
			build2(g[v][i], g[v][i]);
	}
}

int sz;

vector <int> st;
vector <int> add;

void buildst(int tl, int tr, int v = 1) {
	if (tl > tr)
		return;
	add[v] = 0;
	if (tl == tr) {
		if (tl < (int)et.size())
			st[v] = et[tl];
		else
			st[v] = 0;
		return;
	}
	int mid = (tl + tr) / 2;
	buildst(tl, mid, 2 * v);
	buildst(mid + 1, tr, 2 * v + 1);
	st[v] = min(st[2 * v], st[2 * v + 1]);
}

void updst(int l, int r, int delt, int tl = 0, int tr = sz - 1, int v = 1) {
	if (tl > tr)
		return;
	if (tl > r || tr < l)
		return;
	if (tl >= l && tr <= r) {
		add[v] += delt;
		st[v] += delt;
		return;
	}
	int mid = (tl + tr) / 2;
	updst(l, r, delt, tl, mid, 2 * v);
	updst(l, r, delt, mid + 1, tr, 2 * v + 1);
	st[v] = min(st[2 * v], st[2 * v + 1]) + add[v];
}

inline int getadd(int v) {
	int res = 0;
	v /= 2;
	while (v >= 1) {
		res += add[v];
		v /= 2;
	}
	return res;
}

int get_min(int l, int r) {
	int vl = sz + l;
	int vr = sz + r;
	int res = 1e9;
	while (vl <= vr) {
		if (vl & 1) {
			res = min(res, st[vl] + getadd(vl));
			vl++;
		}
		if (vl <= vr && (vr & 1) == 0) {
			res = min(res, st[vr] + getadd(vr));
			vr--;
		}
		vl /= 2;
		vr /= 2;
	}
	return res;
}

/*
int get_min(int l, int r, int tl = 0, int tr = sz - 1, int v = 1) {
	if (tl > tr)
		return 1e9;
	if (tl > r || tr < l)
		return 1e9;
	if (tl >= l && tr <= r)
		return st[v];
	int mid = (tl + tr) / 2;
	return min(get_min(l, r, tl, mid, 2 * v) + add[v], get_min(l, r, mid + 1, tr, 2 * v + 1) + add[v]);
}
*/

void upd_path(int v, int u, int delt) {
	if (dep[v] > dep[u])
		swap(v, u);
	while (dep[h[u]] > dep[v]) {
		updst(sta[h[u]], sta[u], delt);
		u = p[h[u]];
	}
	updst(sta[v], sta[u], delt);
}

int get_min_path(int v, int u) {
	if (dep[v] > dep[u])
		swap(v, u);
	int res = 1e9;
	while (dep[h[u]] > dep[v]) {
		res = min(res, get_min(sta[h[u]], sta[u]));
		u = p[h[u]];
	}
	res = min(res, get_min(sta[v], sta[u]));
	return res;
}

bool ok = true;

long long ans = 0;

int taken[MAXN];

set <pair <int, int>> dfs(int v) {
	set <pair <int, int>> res;
	res.insert({ c[v], v });
	for (auto to : g[v]) {
		auto temp = dfs(to);
		if (temp.size() > res.size())
			temp.swap(res);
		for (auto e : temp)
			res.insert(e);
	}
	int mycnt = 0;
	for (auto to : g[v]) {
		int cnt = r[to] - get_min_path(to, to);
		upd_path(v, v, -cnt);
		mycnt += cnt;
	}
	while (mycnt < l[v] && res.size()) {
		int need = get_min_path(v, res.begin()->second);
		int u = res.begin()->second;
		int cost = res.begin()->first;
		if (need > 0 && need > l[v] - mycnt)
			need = l[v] - mycnt;
		else
			res.erase(res.begin());
		mycnt += need;
		ans += (long long)need * cost;
		taken[u] += need;
		upd_path(u, v, -need);
	}
	if (mycnt < l[v] || mycnt > r[v])
		ok = false;
	return res;
}

void solve() {
	int n;
	cin >> n;
	for (int i = 0; i < n; i++)
		g[i].clear();
	for (int i = 1; i < n; i++) {
		cin >> p[i];
		p[i]--;
		g[p[i]].push_back(i);
	}
	for (int i = 0; i < n; i++)
		cin >> c[i];
	for (int i = 0; i < n; i++)
		cin >> l[i] >> r[i];
	et.clear();
	fill(taken, taken + n, 0);
	build1(0);
	dep[0] = 0;
	build2(0, 0);
	for (int i = 0; i < 19; i++) {
		if ((1 << i) > (int)et.size()) {
			sz = (1 << i);
			break;
		}
	}
	st.resize(2 * sz);
	add.resize(2 * sz);
	buildst(0, sz - 1);
	ok = true;
	ans = 0;
	auto temp = dfs(0);
	if (!ok) {
		cout << "-1\n";
		return;
	}
	cout << ans << "\n";
	for (int i = 0; i < n; i++) {
		cout << taken[i] << " ";
	}
	cout << "\n";
}

int main() {
	ios_base::sync_with_stdio(false);
	cin.tie(0);
	cout.tie(0);
	int tests = 1;
	cin >> tests;
	for (int test = 1; test <= tests; test++) {
		solve();
	}
}