#ifndef LOCAL
#pragma GCC optimize("Ofast")
#pragma GCC optimize("O3")
#endif

#include <iostream>
#include <vector>
#include <algorithm>
#include <queue>
#include <numeric>

#define len(a) (int)a.size()

typedef long long ll;

const int INF = (int)1e9;

using namespace std;

void run();

int main()
{
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
#else
    ios_base::sync_with_stdio(0);
    cin.tie(0);
    cout.tie(0);
#endif

    run();

}

/*struct DSU
{
    vector<int> parent;
    vector<int> rank;
    vector<int> data;

    void init(int size)
    {
        parent.resize(size);
        rank.assign(size, 0);
        data.assign(size, INF);
        iota(parent.begin(), parent.end(), 0);
    }

    int get(int u)
    {
        if (u == parent[u])
        {
            return u;
        }
        return parent[u] = get(parent[u]);
    }

    void merge(int u, int v)
    {
        u = get(u);
        v = get(v);
        if (u == v)
        {
            return;
        }
        if (rank[u] > rank[v])
        {
            swap(u, v);
        }
        if (rank[u] == rank[v])
        {
            ++rank[v];
        }
        parent[u] = v;
    }
};*/

vector< vector<int> > graph;
vector<int> costs;
vector< pair<int, int> > borders;
vector< priority_queue< tuple<int, int>, vector< tuple<int, int> >, greater< tuple<int, int> > > > sts;
vector<int> counts;

struct SegmentTree
{
    vector<int> tree;

    void init(int size)
    {
        tree.assign(4 * size, INF);
    }

    void push(int p, int left, int right)
    {
        if (tree[p] == INF)
        {
            return;
        }
        if (right - left > 1)
        {
            tree[2 * p + 1] = min(tree[2 * p + 1], tree[p]);
            tree[2 * p + 2] = min(tree[2 * p + 2], tree[p]);
            tree[p] = INF;
        }

    }

    void update(int p, int left, int right, int left_, int right_, int value)
    {
        push(p, left, right);
        if ((right <= left_) || (right_ <= left))
        {
            return;
        }
        if ((left_ <= left) && (right <= right_))
        {
            tree[p] = min(tree[p], value);
            return;
        }
        int mid = (left + right) / 2;
        update(2 * p + 1, left, mid, left_, right_, value);
        update(2 * p + 2, mid, right, left_, right_, value);
    }

    int get(int p, int left, int right, int ind)
    {
        push(p, left, right);
        if (right - left == 1)
        {
            return tree[p];
        }
        int mid = (left + right) / 2;
        if (ind < mid)
        {
            return get(2 * p + 1, left, mid, ind);
        }
        return get(2 * p + 2, mid, right, ind);
    }
};

SegmentTree tree;
int euler_timer;
vector<int> first_occur;
vector<int> last_occur;

void dfs_euler(int v)
{
    first_occur[v] = euler_timer++;
    for (int to : graph[v])
    {
        dfs_euler(to);
        euler_timer++;
    }
    last_occur[v] = euler_timer++;
}

pair<int, ll> dfs(int v)
{
    int sum_count;
    ll sum_cost;
    sum_cost = sum_count = 0;
    int max_sts = v;
    for (int to : graph[v])
    {
        auto [count, cost] = dfs(to);
        if (count == -1)
        {
            return {-1, -1};
        }
        sum_count += count;
        sum_cost += cost;
        if (len(sts[to]) > len(sts[max_sts]))
        {
            max_sts = to;
        }
    }
    if (sum_count > borders[v].second)
    {
        return {-1, -1};
    }
    for (int to : graph[v])
    {
        if (to == max_sts)
        {
            continue;
        }
        while (!sts[to].empty())
        {
            sts[max_sts].push(sts[to].top());
            sts[to].pop();
        }
    }
    tree.update(0, 0, euler_timer, first_occur[v], last_occur[v] + 1, borders[v].second - sum_count);
    sts[max_sts].push({costs[v], v});
    swap(sts[max_sts], sts[v]);
    while (sum_count < borders[v].first)
    {
        auto [cost, i] = sts[v].top();
        sts[v].pop();
        int left = tree.get(0, 0, euler_timer, first_occur[i]);
        int spend = min(borders[v].first - sum_count, left);
        sum_count += spend;
        left -= spend;
        counts[i] += spend;
        sum_cost += (ll)spend * cost;
        if (left)
        {
            sts[v].push({cost, i});
        }
        tree.update(0, 0, euler_timer, first_occur[i], last_occur[i] + 1, left);
        tree.update(0, 0, euler_timer, first_occur[v], last_occur[v] + 1, borders[v].second - sum_count);
    }

    return {sum_count, sum_cost};
}

void solve_test()
{
    int n;
    cin >> n;
    graph.assign(n, vector<int>());
    for (int i = 1; i < n; ++i)
    {
        int parent;
        cin >> parent;
        graph[parent - 1].push_back(i);
    }
    costs.resize(n);
    borders.resize(n);
    for (int i = 0; i < n; ++i)
    {
        cin >> costs[i];
    }
    for (int i = 0; i < n; ++i)
    {
        cin >> borders[i].first >> borders[i].second;
    }

    euler_timer = 0;
    first_occur.resize(n);
    last_occur.resize(n);
    dfs_euler(0);
    tree.init(euler_timer);

    sts.assign(n, priority_queue< tuple<int, int>, vector< tuple<int, int> >, greater< tuple<int, int> > >());
    counts.assign(n, 0);
    auto [count, cost] = dfs(0);
    if (count != -1)
    {
        cout << cost << '\n';
        for (int i : counts)
        {
            cout << i << ' ';
        }
        cout << '\n';
    }
    else
    {
        cout << "-1\n";
    }

}

void run()
{
    int t;
    cin >> t;
    for (; t > 0; --t)
    {
        solve_test();
    }
}
