#include <iostream>
#include <random>
#include <vector>
#include <climits>
#include <map>
#include <algorithm>
std::vector<std::vector<int>> children;
std::vector<int> parent, cost_here, lcons, rcons;
std::vector<int64_t> ansv;
std::vector<int> bought, tbought;
int64_t gans = 0;
bool FAIL = false;
struct cmp
{
	bool operator()(int a, int b) const
	{
		return std::tie(cost_here[a], a) < std::tie(cost_here[b], b);
	}
};
std::pair<int,int> min_cost(int at)
{
	if(tbought[at] >= rcons[at]) return { INT_MAX, -1 } ; // can't buy more
	int c = cost_here[at], i = at;
	for(int to : children[at])
	{
		auto[c2,i2] = min_cost(to);
		if(c2 < c) c = c2, i = i2;
	}
	return {c, i};
}
bool dfs2(int at, int tgt)
{
	if(at == tgt) { tbought[at]++; return 1; }
	for(int to : children[at])
	{
		if(dfs2(to, tgt)) { tbought[at]++; return 1; }
	}
	return 0;
}

bool buy1(int at)
{
	// dfs over subtree, find min cost
	auto[cost, from] = min_cost(at);
	if(from == -1) return 0;
	bought[from]+=1;
	//printf("buying from %d: %d\n", from, cost_here[from]);
	gans += cost_here[from];
	// increase tbought on the path to that vertex
	dfs2(0, from);
	//printf("after buying at %d: ", at); for(int x : tbought) printf("%d ", x); printf("\n");
	return 1;
}
void dfs(int at)
{
	//int n_extra = rcons[at], n_rem = lcons[at];
	for(int to : children[at])
	{
		dfs(to);
		if(FAIL) return;
		//n_rem -= lcons[to];
	}
	int n_rem = lcons[at] - tbought[at];
	if(tbought[at] > rcons[at]) { FAIL = true; return; }
	// buy as cheaply as possible from the subtree
	// recalculate constraints (compute `bought` for subtrees, if it's ==rcons skip the subtree)
	while(n_rem > 0)
	{
		if(!buy1(at)) { FAIL = true; return; }
		n_rem--;
	}
}
int64_t very_dumb_solve()
{
	// try all tuples
	int n =children.size();
	int64_t end = 1;
	for(int i = 0; i < n; i++) end *= rcons[0];
	int64_t gans = LLONG_MAX;
	for(int64_t m = 0; m < end; m++)
	{
		int64_t cm =m ;
		//printf("m=%lld\n", m);
		int64_t ans = 0;
		std::vector<int> tbought(n);
		std::vector<int> bought(n);
		for(int j = 0; j < n; j++)
		{
			int here = cm % rcons[0];
			cm /= rcons[0];
			bought[j] += here;
			ans += here * 1ll * cost_here[j];
			int cur = j;
			while(cur != -1) tbought[cur] += here, cur = parent[cur];
		}
		bool ok = true;
		for(int j = 0; j < n; j++)
		{
			if(tbought[j] < lcons[j] || tbought[j] > rcons[j]) ok = false;
		}
		if(ok)
		{
			for(int x : bought) printf("%d ", x);
			printf("\n");
			gans = std::min(gans, ans);
		}
	}
	return gans == LLONG_MAX ? -1 : gans;
}
std::mt19937 mt;
int main()
{
	// minimize total cost if l <= sum in each subtree <= r
	// solution exists iff buying minimally to satisfy l in each vertex won't violate any r constraints
	// buying higher in the tree is always acceptable, if it's cheaper and won't violate child constraints
	// buy as little as possible in leaves
	// when merging trees, if there is not enough, buy from the cheapest vertices in the subtree
	// return counter maps (cost, n_remaining), add them for each child
	int T; std::cin >> T;
	while(T --> 0)
	{
		int n; std::cin >> n;
		gans = 0;
		FAIL = false;
		children.assign(n, {});
		parent.assign(n, {});
		cost_here.assign(n, {});
		lcons.assign(n, {});
		rcons.assign(n, {});
		ansv.assign(n, {});
		bought.assign(n, {});
		tbought.assign(n, {});
		parent[0] = -1;
		for(int i = 0; i < n-1; i++)
		{
			int p; scanf("%d", &p); p--;
			parent[i+1] = p;
			children[p].push_back(i+1);
		}
		for(int i = 0; i < n; i++) scanf("%d", &cost_here[i]);
		//for(int i = 0; i < n; i++) cost_here[i] = mt() % 10 + 1;
		for(int i = 0; i < n; i++) scanf("%d%d", &lcons[i], &rcons[i]);
		//for(int i = 0; i < n; i++)
		//{
		//	lcons[i] = mt() % 3 + 1;
		//	rcons[i] = lcons[i] + mt() % 2;
		//}
		//for(int i = 1; i < n; i++)
		//	lcons[parent[i]] += lcons[i], rcons[parent[i]] += rcons[i];
		dfs(0);
		if(!FAIL)
		{
			printf("%lld\n", gans);
			for(int x : bought) printf("%d ", x);
			printf("\n");
		}
		else printf("-1\n");
		//printf("%lld\n", very_dumb_solve());
	}
}
