#include <iostream>
#include <vector>
#include <set>

using namespace std;

typedef long long ll;

const int INF = int(1e9) + 7;

int n;
vector<int> p, c, l, r, sz, tin, tout, top;
vector<vector<int>> gr;

void dfs_sz(int v) {
	sz[v] = 1;
	for (int& u : gr[v]) {
		dfs_sz(u);
		sz[v] += sz[u];
		if (sz[u] > sz[gr[v][0]]) {
			swap(u, gr[v][0]);
		}
	}
}

int timer = 0;
void dfs_timer(int v, int tp) {
	tin[v] = timer++;
	// cout << "tin[" << v << "] = " << tin[v] << "\n";
	top[v] = tp;
	for (int i = 0; i < int(gr[v].size()); i++) {
		int u = gr[v][i];
		dfs_timer(u, (i == 0 ? tp : u));
	}
	tout[v] = timer;
}

bool is_p(int p, int v) {
	return (tin[p] <= tin[v] && tout[p] >= tout[v]);
}

struct node {
	ll min, push;
	node(): min(INF), push(0) {}
};

int b;
vector<node> tree;

void build_tree() {
	b = 1;
	while (b < n) {
		b *= 2;
	}
	// cout << "b = " << b << "\n";
	tree.assign(2 * b, node());
	for (int i = 0; i < n; i++) {
		// cout << "tin[" << i << "] = " << tin[i] << "\n";
		tree[b + tin[i]].min = r[i];
	}
	for (int t = b - 1; t > 0; t--) {
		tree[t].min = min(tree[t * 2].min, tree[t * 2 + 1].min);
	}
}

void set_push(int t, ll val) {
	tree[t].min -= val;
	tree[t].push += val;
}

void push(int t) {
	if (!tree[t].push) {
		return;
	}
	set_push(t * 2, tree[t].push);
	set_push(t * 2 + 1, tree[t].push);
	tree[t].push = 0;
}

void update(int t) {
	tree[t].min = min(tree[t * 2].min, tree[t * 2 + 1].min);
}

void substract(int t, int tl, int tr, int left, int right, int val) {
	if (left >= right) {
		return;
	}
	if (tl == left && tr == right) {
		set_push(t, val);
		return;
	}
	push(t);
	int mid = (tl + tr) / 2;
	substract(t * 2, tl, mid, left, min(right, mid), val);
	substract(t * 2 + 1, mid, tr, max(left, mid), right, val);
	update(t);
}

int get_min(int t, int tl, int tr, int left, int right) {
	if (left >= right) {
		return INF;
	}
	if (tl == left && tr == right) {
		return tree[t].min;
	}
	push(t);
	int mid = (tl + tr) / 2, ans = INF;
	ans = min(ans, get_min(t * 2, tl, mid, left, min(right, mid)));
	ans = min(ans, get_min(t * 2 + 1, mid, tr, max(left, mid), right));
	return ans;
}

void substract_vertex(int v, int cnt) {
	while (v != -1) {
		substract(1, 0, b, tin[top[v]], tin[v] + 1, cnt);
		v = p[top[v]];
	}
}

int get_max_free(int v) {
	int min_val = INF;
	while (v != -1) {
		min_val = min(min_val, get_min(1, 0, b, tin[top[v]], tin[v] + 1));
		v = p[top[v]];
	}
	return min_val;
}

struct ans {
	int cnt;
	ll sum;
	set<pair<int, int>>* s;
	ans(): cnt(-1), sum(-1) {}
};

void unite(set<pair<int, int>>*& s1, set<pair<int, int>>* s2) {
	if (s1->size() < s2->size()) {
		swap(s1, s2);
	}
	for (const auto& el : *s2) {
		s1->insert(el);
	}
}

ans dfs_ans(int v) {
	int cnt = 0;
	ll sum = 0;
	set<pair<int, int>>* s = new set<pair<int, int>>();
	s->insert({ c[v], v });
	for (int u : gr[v]) {
		auto res = dfs_ans(u);
		if (res.cnt == -1) {
			return ans();
		}
		cnt += res.cnt;
		sum += res.sum;
		unite(s, res.s);
	}
	while (cnt < l[v]) {
		if (s->empty()) {
			return ans();
		}
		int u = s->begin()->second;
		int add = min(l[v] - cnt, get_max_free(u));
		if (add == 0) {
			s->erase(s->begin());
			continue;
		}
		substract_vertex(u, add);
		if (add < l[v] - cnt) {
			s->erase(s->begin());
		}
		cnt += add;
		sum += add * 1ll * c[u];
	}
	if (cnt > r[v]) {
		return ans();
	}
	ans res;
	res.cnt = cnt;
	res.sum = sum;
	res.s = s;
	return res;
}

vector<ll> sum;
vector<int> a;

void dfs_sum(int v) {
	ll res = sum[v];
	for (int u : gr[v]) {
		dfs_sum(u);
		res -= sum[u];
	}
	a[v] = res;
}

void solve() {
	cin >> n;
	p.resize(n), c.resize(n), l.resize(n), r.resize(n);
	gr.assign(n, vector<int>());
	p[0] = -1;
	for (int i = 1; i < n; i++) {
		cin >> p[i];
		p[i]--;
		gr[p[i]].push_back(i);
		// cout << p[i] << " -> " << i << "\n";
		// cout << "sz = " << gr[p[i]].size() << "\n";
	}
	for (int i = 0; i < n; i++) {
		cin >> c[i];
	}
	for (int i = 0; i < n; i++) {
		cin >> l[i] >> r[i];
	}
	sz.assign(n, 0);
	dfs_sz(0);
	tin.resize(n), tout.resize(n), top.resize(n);
	timer = 0;
	dfs_timer(0, 0);
	build_tree();
	auto res = dfs_ans(0);
	if (res.cnt == -1) {
		cout << -1 << "\n";
		return;
	}
	cout << res.sum << "\n";
	for (int t = 1; t < b; t++) {
		push(t);
	}
	sum.resize(n);
	for (int i = 0; i < n; i++) {
		sum[i] = r[i] - tree[b + tin[i]].min;
	}
	a.resize(n);
	dfs_sum(0);
	for (int i = 0; i < n; i++) {
		cout << a[i] << " ";
	}
	cout << "\n";
}

int main() {
	int t;
	cin >> t;
	for (int i = 0; i < t; i++) {
		solve();
	}
}
