#include <iostream>
#include <vector>
#include <algorithm>
#include <set>
#include <map>
#include <stack>
#include <deque>
#include <cmath>
#include <random>
#include <iomanip>

#pragma GCC target("avx,avx2,sse,sse2,sse3,sse4,popcnt")
#pragma GCC optimize("-Ofast,unroll-loops,fast-math")

using namespace std;

mt19937 rnd(time(0));

class persst {
private:
	struct node {
		int val = 0, lft = -1, rgh = -1;
		node(int _v = 0) : val(_v) {}
	};
	vector <node> nodes;
	vector <int> versions;
	void build(int ind, int l, int r, vector <int>& v) {
		if (ind >= nodes.size())
			nodes.emplace_back();
		if (l == r) {
			nodes[ind].val = v[l];
			return;
		}
		int mid = (l + r) / 2;
		nodes[ind].lft = nodes.size();
		build(nodes.size(), l, mid, v);
		nodes[ind].rgh = nodes.size();
		build(nodes.size(), mid + 1, r, v);
		nodes[ind].val = nodes[nodes[ind].lft].val + nodes[nodes[ind].rgh].val;
	}
	int val(int ind, int l, int r, int i) {
		if (l == r)
			return nodes[ind].val;
		int mid = (l + r) / 2;
		if (mid >= i)
			return val(nodes[ind].lft, l, mid, i);
		else
			return val(nodes[ind].rgh, mid + 1, r, i);
	}
	int add(int ind, int l, int r, int i) {
		if (l == r) {
			int ind2 = nodes.size();
			nodes.emplace_back(1);
			return ind2;
		}
		int ind2 = nodes.size();
		nodes.push_back(nodes[ind]);
		++nodes[ind2].val;
		int mid = (l + r) / 2;
		if (mid >= i) {
			nodes[ind2].lft = add(nodes[ind].lft, l, mid, i);
			return ind2;
		}
		nodes[ind2].rgh = add(nodes[ind].rgh, mid + 1, r, i);
		return ind2;
	}
	int sum(int ind, int l, int r, int nl, int nr) {
		if (nl == l && nr == r) {
			return nodes[ind].val;
		}
		int mid = (nl + nr) / 2;
		if (mid >= r)
			return sum(nodes[ind].lft, l, r, nl, mid);
		else if (mid < l)
			return sum(nodes[ind].rgh, l, r, mid + 1, nr);
		return sum(nodes[ind].lft, l, mid, nl, mid) + sum(nodes[ind].rgh, mid + 1, r, mid + 1, nr);
	}
	int n = 0;
public:
	persst() {};
	void init(int _n, int m, vector <int>& v) {
		n = _n;
		nodes.reserve(4 * n + 20 * m);
		build(0, 0, n - 1, v);
		versions.push_back(0);
	}
	void add(int ind, int version) {
		versions.push_back(add(versions[version], 0, n - 1, ind));
	}
	int sum(int version, int l, int r) {
		return sum(versions[version], l, r, 0, n - 1);
	}
};

int n, q;

vector <int> a, v;

persst pst;

int find_instr(int c, int r) {
	int lft = -1, rgh = n + 1, mid;
	while (rgh - lft > 1) {
		mid = (lft + rgh) / 2;
		if (pst.sum(mid, 0, r) + c < r)
			lft = mid;
		else
			rgh = mid;
	}
	return rgh;
}

int main() {
	cin >> n;
	a.resize(n);
	v.resize(n + 1);
	for (int i = 0; i < n; ++i) {
		cin >> a[i];
	}
	cin >> q;
	pst.init(n + 1, n, v);
	for (int i = 0; i < n; ++i)
		pst.add(a[i], i);
	int c1, r1, c2, r2;
	for (int i = 0; i < q; ++i) {
		cin >> r1 >> c1 >> r2 >> c2;
		if (r1 == 0 || r2 == 0) {
			cout << "0 0\n";
			continue;
		}
		int k1 = find_instr(c1, r1), k2 = find_instr(c2, r2);
		int lft = 0, rgh = min(r1, r2) + 1, mid;
		while (rgh - lft > 1) {
			mid = (lft + rgh) / 2;
			if (pst.sum(k1, 0, mid) != pst.sum(k2, 0, mid))
				rgh = mid;
			else
				lft = mid;
		}
		int rr = lft;
		int cc = rr;
		cc -= pst.sum(k1, 0, lft);
		cout << rr << " " << cc << "\n";
	}
}
