#include <iostream>
#include <vector>
#include <algorithm>
#include <string>
#include <set>
#include <stack>
#include <map>
#include <cstdlib>
#include <cassert>

using namespace std;
#pragma GCC optimize("03")

vector<int> gol;
vector<int> gor;
vector<int> d;
vector<vector<int>> ppow;

void godwn(const vector<bool>& wayr) {
    int crr = 0;
    int crc = 0;
    for (int i = 0; i < wayr.size(); i++) {
        int cri = (crr * (crr + 1)) / 2 + crc;
        if (wayr[i]) {
            // right
            int tor = crr + 1;
            int toc = crc + 1;
            if (gor[cri] == -1) {
                gor[cri] = ((tor * (tor + 1)) / 2 + toc);
            }
            crr = tor;
            crc = toc;
        } else {
            // left
            int tor = crr + 1;
            int toc = crc;
            if (gol[cri] == -1) {
                gol[cri] = ((tor * (tor + 1)) / 2 + toc);
            }
            crr = tor;
            crc = toc;
        }
    }
}

void dfsbuild(int crr, int crc) {
    int cri = (crr * (crr + 1)) / 2 + crc;
    for (int p = 1; p < 20; p++) {
        int frm = ppow[cri][p - 1];
        if (frm != -1) {
            ppow[cri][p] = ppow[frm][p - 1];
        }
    }
    if (gor[cri] != -1) {
        //right
        int tor = crr + 1;
        int toc = crc + 1;
        int toi = (tor * (tor + 1) / 2) + toc;
        ppow[toi][0] = cri;
        d[toi] = d[cri] + 1;
        dfsbuild(crr + 1, crc + 1);
    }
    if (gol[cri] != -1) {
        //left
        int tor = crr + 1;
        int toc = crc;
        int toi = (tor * (tor + 1) / 2) + toc;
        ppow[toi][0] = cri;
        d[toi] = d[cri] + 1;
        dfsbuild(crr + 1, crc);
    }
}

pair<int, int> getindbyi(int cri) {
    if (cri == 0) return {0, 0};
    int l = 0;
    int r = 3200;
    while (r - l > 1) {
        int m = (l + r) / 2;
        if (m * (m + 1) / 2 <= cri) {
            l = m;
        } else {
            r = m;
        }
    }
    return {l, cri - (l * (l + 1) / 2)};
}

int getlca(int a, int b) {
    if (d[a] > d[b]) swap(a, b);
    for (int p = 19; p >= 0; p--) {
        int tob = ppow[b][p];
        if (tob != -1 && d[tob] >= d[a]) b = tob;
    }
    if (a == b) return a;
    for (int p = 19; p >= 0; p--) {
        int toa = ppow[a][p];
        int tob = ppow[b][p];
        if (toa != tob) {
            a = toa;
            b = tob;
        }
    }
    return ppow[a][0];
}

signed main() {
    int n;
    cin >> n;
    n++;
    d.resize(n * (n + 1) / 2, 0);
    gol.resize(n * (n + 1) / 2, -1);
    gor.resize(n * (n + 1) / 2, -1);
    ppow.resize(n * (n + 1) / 2, vector<int>(20, -1));
    vector<bool> wayr(n - 1, true);
    godwn(wayr);
    for (int i = 0; i < n - 1; i++) {
        int v;
        cin >> v;
        v--;
        wayr[v] = false;
        godwn(wayr);
    }
    dfsbuild(0, 0);
    int q;
    cin >> q;
    while (q--) {
        int r1, c1, r2, c2;
        cin >> r1 >> c1 >> r2 >> c2;
        int i1 = (r1 * (r1 + 1)) / 2 + c1;
        int i2 = (r2 * (r2 + 1)) / 2 + c2;
        int i3 = getlca(i1, i2);
        auto pp = getindbyi(i3);
        cout << pp.first << ' ' << pp.second << '\n';
    }
    return 0;
}
