#include <iostream>
#include <algorithm>
#include <set>
#include <random>
#include <map>

struct Segtree {
    int32_t* data;
    int32_t size;

    Segtree(int32_t n) {
        data = new int32_t[n + 1];
        size = n;

        for(int32_t i = 0; i <= n; i++)
            data[i] = 0;
    }

    void add(int32_t ind, int32_t val) {
        ind++;
        for(; ind <= size; ind += ind & (-ind))
            data[ind] += val;
    }

    int32_t get(int32_t ind) {
        int32_t res = 0;
        for(; ind > 0; ind -= ind & (-ind))
            res += data[ind];
        return res;
    }
};

struct SparseTable {
    int32_t** table;

    SparseTable(int32_t n, int32_t* arr) {
        int32_t max_level = 0;
        while((1 << max_level) <= n)
            max_level++;
        max_level--;

        table = new int32_t*[max_level + 1];
        for(int32_t i = 0; i <= max_level; i++) {
            table[i] = new int32_t[n - (1 << i) + 1];
            for(int32_t j = 0; j < n - (1 << i) + 1; j++)
                if(i == 0)
                    table[i][j] = arr[j];
                else
                    table[i][j] = std::min(table[i - 1][j], table[i - 1][j + (1 << i) / 2]);
        }
    }

    int32_t get(int32_t left, int32_t right) {
        int32_t level = 31 - __builtin_clz(right - left + 1);
        return std::min(table[level][left], table[level][right - (1 << level) + 1]);
    }
};

struct Node {
    Node* left, *right;
    int32_t sum;
};

int32_t get_sum(Node* node, int32_t left, int32_t right, int32_t query_left, int32_t query_right) {
    if(left >= query_right || right <= query_left)
        return 0;
    if(left >= query_left && right <= query_right)
        return node->sum;
    int32_t m = (left + right) / 2;
    return get_sum(node->left, left, m, query_left, query_right) + get_sum(node->right, m, right, query_left, query_right);
}

Node* update(Node* node, int32_t left, int32_t right, int32_t ind) {
    Node* res = new Node;
    res->sum = node->sum + 1;
    if (right - left == 1) {
        return res;
    }
    int32_t m = (left + right) / 2;
    if(ind >= m) {
        res->left = node->left;
        res->right = update(node->right, m, right, ind);
    } else {
        res->right = node->right;
        res->left = update(node->left, left, m, ind);
    }
    return res;
}

Node* build(int32_t left, int32_t right) {
    Node* res = new Node;
    res->sum = 0;
    if(right - left == 1)
        return res;
    int32_t m = (left + right) / 2;
    res->left = build(left, m);
    res->right = build(m, right);
    return res;
}

int32_t n;
int32_t* arr;
Node** roots;
int32_t position_on_level(int32_t prefix, int32_t level) {
    /*int32_t ans = level;
    for(int32_t i = 0; i < prefix; i++)
        if(arr[i] < level)
            ans--;
    return ans;*/
    return level - get_sum(roots[prefix], 0, n, 0, level);
}

std::map<std::pair<int32_t, int32_t>, int32_t> version_ans;
void solve(int32_t left, int32_t right, std::vector<std::pair<int32_t, int32_t> >& queries,
           Segtree& segtree, int32_t& cur_version) {
    if(right - left == 1) {
        for (auto p: queries)
            version_ans[p] = right;
        return;
    }
    int32_t m = (left + right) / 2;
    while(cur_version < m) {
        segtree.add(arr[cur_version], 1);
        cur_version++;
    }
    while(cur_version > m) {
        segtree.add(arr[cur_version - 1], -1);
        cur_version--;
    }
    std::vector<std::pair<int32_t, int32_t>> go_right, go_left;
    for(int32_t i = 0; i < queries.size(); i++)
        if(queries[i].first - segtree.get(queries[i].first) > queries[i].second)
            go_right.push_back(queries[i]);
        else
            go_left.push_back(queries[i]);
    solve(m, right, go_right, segtree, cur_version);
    solve(left, m, go_left, segtree, cur_version);
}

struct Query {
    int32_t r1, c1, r2, c2;

    Query(){}

    Query(int32_t r1_, int32_t c1_, int32_t r2_, int32_t c2_) {
        r1 = r1_;
        c1 = c1_;
        r2 = r2_;
        c2 = c2_;
    }
};

int main() {
    std::ios::sync_with_stdio(false);
    std::cin.tie(0);

    std::cin >> n;

    std::mt19937 rng;

    arr = new int32_t[n];
    for(int32_t i = 0; i < n; i++) {
        std::cin >> arr[i];
        //arr[i] = i + 1;
        arr[i]--;
    }

    roots = new Node*[n + 1];
    roots[0] = build(0, n);
    for(int32_t i = 1; i <= n; i++) {
        roots[i] = update(roots[i - 1], 0, n, arr[i - 1]);
    }

    SparseTable table(n, arr);

    int32_t num_q;
    std::cin >> num_q;

    Query* queries = new Query[num_q];
    std::vector<std::pair<int32_t, int32_t> > points;
    for(int32_t q = 0; q < num_q; q++) {
        int32_t r1, c1, r2, c2;
        std::cin >> r1 >> c1 >> r2 >> c2;

        points.emplace_back(r1, c1);
        points.emplace_back(r2, c2);
        queries[q] = {r1, c1, r2, c2};
    }

    Segtree segtree(n);
    int32_t cur_version = 0;
    solve(-1, n, points, segtree, cur_version);

    for(int32_t q = 0; q < num_q; q++) {
        int32_t r1 = queries[q].r1, c1 = queries[q].c1, r2 = queries[q].r2, c2 = queries[q].c2;

        int32_t v1 = version_ans[{r1, c1}];
        int32_t v2 = version_ans[{r2, c2}];

        int32_t old_v1 = v1, old_v2 = v2;
        if(v1 > v2)
            std::swap(v1, v2);
        if(v1 == v2) {
            std::cout << (r1 < r2 ? r1 : r2) << " " << (r1 < r2 ? c1 : c2) << "\n";
            continue;
        }
        if(old_v1 < old_v2 && r1 < r2 && position_on_level(old_v2, r1) == c1) {
            std::cout << r1 << " " << c1 << "\n";
            continue;
        }
        if(old_v2 < old_v1 && r2 < r1 && position_on_level(old_v1, r2) == c2) {
            std::cout << r2 << " " << c2 << "\n";
            continue;
        }

        int32_t high_update = table.get(v1, v2 - 1);
        int32_t ans_level = high_update;
        std::cout << ans_level << " " << position_on_level(v1, ans_level) << "\n";
    }
    return 0;
}
