#include <set>
#include <vector>
#include <cassert>
#include <iomanip>
#include <iostream>
#include <algorithm>

using namespace std;

const int MAXN = 100'100;
int sz[MAXN];
vector<int> G[MAXN];
int up[MAXN];

void dfs_sz(int v) {
  sz[v] = 1;
  for (int i = 0; i < G[v].size(); ++i) {
    dfs_sz(G[v][i]);
    sz[v] += sz[G[v][i]];
    if (sz[G[v][i]] > sz[G[v][0]])
      swap(G[v][i], G[v][0]);
  }
}

vector<int> order;
int tin[MAXN];
int path_id[MAXN];
int path_st[MAXN];

void dfs(int v) {
  tin[v] = order.size();
  order.emplace_back(v);
  for (int u : G[v])
    dfs(u);
  if (G[v].empty()) {
    path_id[v] = v;
    path_st[path_id[v]] = v;
  } else {
    path_id[v] = path_id[G[v][0]];
    path_st[path_id[v]] = v;
  }
}

int tree[(1 << 18) + 228];
int mod[(1 << 18) + 228];
const int INF = 1e9 + 7;

void build(int v, int left, int right) {
  tree[v] = 0;
  mod[v] = 0;
  if (left + 1 == right)
    return;
  int mid = (left + right) / 2;
  build(2 * v + 1, left, mid);
  build(2 * v + 2, mid, right);
}

void add(int v, int left, int right, int x, int y, int d) {
  if (y <= left || right <= x)
    return;
  if (x <= left && right <= y) {
    tree[v] += d;
    mod[v] += d;
    return;
  }
  int mid = (left + right) / 2;
  add(2 * v + 1, left, mid, x, y, d);
  add(2 * v + 2, mid, right, x, y, d);
  tree[v] = min(tree[2 * v + 1], tree[2 * v + 2]) + mod[v];
}

int get(int v, int left, int right, int x, int y) {
  if (y <= left || right <= x)
    return INF;
  if (x <= left && right <= y)
    return tree[v];
  int mid = (left + right) / 2;
  return min(get(2 * v + 1, left, mid, x, y), get(2 * v + 2, mid, right, x, y)) + mod[v];
}

long long ll[MAXN];
long long rr[MAXN];
int c[MAXN];
int ans[MAXN];
long long res;
bool ok;
int id[MAXN];
set<pair<int, int>> S[MAXN];
int n;

long long dfs_dp(int v) {
  id[v] = v;
  S[v].emplace(c[v], v);
  long long have = 0;
  for (int u: G[v]) {
    have += dfs_dp(u);
    if (S[id[u]].size() > G[id[v]].size())
      swap(id[u], id[v]);
    for (auto t : S[id[u]])
      S[id[v]].emplace(t);
  }
  if (have > rr[v]) {
    ok = false;
    return 0;
  }
  add(0, 0, n, tin[v], tin[v] + 1, rr[v] - have);
  while (have < ll[v]) {
    if (S[id[v]].empty()) {
      ok = false;
      return 0;
    }
    int u = S[id[v]].begin()->second;
    int val = INF;
    int uu = u;
    while (path_id[uu] != path_id[v]) {
      val = min(val, get(0, 0, n, tin[path_st[path_id[uu]]], tin[uu] + 1));
      uu = up[path_st[path_id[uu]]];
    }
    val = min(val, get(0, 0, n, tin[v], tin[uu] + 1));
    if (val <= ll[v] - have)
      S[id[v]].erase(S[id[v]].begin());
    else
      val = ll[v] - have;
    have += val;
    res += val * 1ll * c[u];
    ans[u] += val;
    uu = u;
    while (path_id[uu] != path_id[v]) {
      add(0, 0, n, tin[path_st[path_id[uu]]], tin[uu] + 1, -val);
      uu = up[path_st[path_id[uu]]];
    }
    add(0, 0, n, tin[v], tin[uu] + 1, -val);
  }
  return have;
}

int main() {
#ifdef DEBUG
  freopen("input.txt", "r", stdin);
  freopen("output.txt", "w", stdout);
#else
  ios::sync_with_stdio(false);
  cin.tie(0);
  cout.tie(0);
#endif

  int t;
  cin >> t;
  while (t--) {
    cin >> n;
    for (int i = 1; i < n; ++i) {
      int p;
      cin >> p;
      up[i] = p - 1;
      G[p - 1].emplace_back(i);
    }
    for (int i = 0; i < n; ++i)
      cin >> c[i];
    for (int i = 0; i < n; ++i)
      cin >> ll[i] >> rr[i];
    dfs_sz(0);
    dfs(0);
    res = 0;
    ok = true;
    fill(ans, ans + n, 0);
    dfs_dp(0);
    if (ok) {
      cout << res << '\n';
      for (int i = 0; i < n; ++i)
        cout << ans[i] << ' ';
      cout << '\n';
    } else {
      cout << -1 << '\n';
    }
    for (int i = 0; i < n; ++i)
      G[i].clear();
    order.clear();
  }
  return 0;
}
