#include <vector>
#include <cassert>
#include <iomanip>
#include <iostream>
#include <algorithm>

using namespace std;

struct Node{
  int val;
  int left, right;
  Node(int _val) {
    val = _val;
    left = right = -1;
  }
  Node() {
  }
};

Node nodes[10'000'000];
int nodes_cnt = 0;

int node(int val) {
  nodes[nodes_cnt] = Node(val);
  ++nodes_cnt;
  return nodes_cnt - 1;
}

int build(int left, int right) {
  int ans = node(0);
  if (left + 1 == right)
    return ans;
  int mid = (left + right) / 2;
  nodes[ans].left = build(left, mid);
  nodes[ans].right = build(mid, right);
  return ans;
}

int add(int v, int left, int right, int i, int x) {
  int ans = node(nodes[v].val + x);
  if (left + 1 == right)
    return ans;
  nodes[ans].left = nodes[v].left;
  nodes[ans].right = nodes[v].right;
  int mid = (left + right) / 2;
  if (i < mid)
    nodes[ans].left = add(nodes[v].left, left, mid, i, x);
  else
    nodes[ans].right = add(nodes[v].right, mid, right, i, x);
  return ans;
}

int get_at_least(int v, int left, int right, int k) {
  if (nodes[v].val < k)
    return right + 1;
  if (left + 1 == right)
    return left + 1;
  int mid = (left + right) / 2;
  int ans = get_at_least(nodes[v].left, left, mid, k);
  if (ans == mid + 1)
    return get_at_least(nodes[v].right, mid, right, k - nodes[nodes[v].left].val);
  return ans;
}

int get_sum(int v, int left, int right, int x, int y) {
  if (y <= left || right <= x)
    return 0;
  if (x <= left && right <= y)
    return nodes[v].val;
  int mid = (left + right) / 2;
  return get_sum(nodes[v].left, left, mid, x, y) + get_sum(nodes[v].right, mid, right, x, y);
}

const int MAXN = 500'500;
const int K = 20;
int table[MAXN][K];
int lg[MAXN];

int get(int l, int r) {
  if (l == r)
    return MAXN;
  int k = lg[r - l];
  return min(table[l][k], table[r - (1 << k)][k]);
}

int main() {
#ifdef DEBUG
  freopen("input.txt", "r", stdin);
  freopen("output.txt", "w", stdout);
#else
  ios::sync_with_stdio(false);
  cin.tie(0);
  cout.tie(0);
#endif

  lg[0] = -1;
  for (int i = 1; i < MAXN; ++i)
    lg[i] = lg[i / 2] + 1;

  int n;
  cin >> n;
  vector<int> A(n);
  vector<int> id(n);
  for (int i = 0; i < n; ++i) {
    cin >> A[i];
    --A[i];
    id[A[i]] = i;
  }
  for (int i = 0; i < n; ++i)
    table[i][0] = A[i];
  for (int j = 1; j < K; ++j) {
    for (int i = 0; i + (1 << j) <= n; ++i)
      table[i][j] = min(table[i][j - 1], table[i + (1 << (j - 1))][j - 1]);
  }
  vector<int> trees(n + 1);
  trees[0] = build(0, n);
  for (int i = 0; i < n; ++i)
    trees[i + 1] = add(trees[i], 0, n, id[i], 1);
  int q;
  cin >> q;
  while (q--) {
    int r1, c1, r2, c2;
    cin >> r1 >> c1 >> r2 >> c2;
    if (r2 > r1) {
      swap(r1, r2);
      swap(c1, c2);
    }
    int i1 = (r1 == c1 ? 0 : get_at_least(trees[r1], 0, n, r1 - c1));
    int i2 = (r2 == c2 ? 0 : get_at_least(trees[r2], 0, n, r2 - c2));
    int row = min(min(r1, r2), get(min(i1, i2), max(i1, i2)));
    cout << row << ' ' << row - get_sum(trees[row], 0, n, 0, i1) << '\n';
  }
  return 0;
}
