#include <vector>
#include <cassert>
#include <iomanip>
#include <iostream>
#include <algorithm>

using namespace std;

int tree[(1 << 20) + 228];

void add(int v, int left, int right, int i, int x) {
  tree[v] += x;
  if (left + 1 == right)
    return;
  int mid = (left + right) / 2;
  if (i < mid)
    add(2 * v + 1, left, mid, i, x);
  else
    add(2 * v + 2, mid, right, i, x);
}

int get_at_least(int v, int left, int right, int k) {
  if (tree[v] < k)
    return right + 1;
  if (left + 1 == right)
    return left + 1;
  int mid = (left + right) / 2;
  int ans = get_at_least(2 * v + 1, left, mid, k);
  if (ans == mid + 1)
    return get_at_least(2 * v + 2, mid, right, k - tree[2 * v + 1]);
  return ans;
}

const int MAXN = 500'500;
const int K = 20;
int table[MAXN][K];
int lg[MAXN];

int get(int l, int r) {
  if (l == r)
    return MAXN;
  int k = lg[r - l];
  return min(table[l][k], table[r - (1 << k)][k]);
}

int fenw[MAXN];

void add(int i, int x) {
  for (; i < MAXN; i = (i | (i + 1)))
    fenw[i] += x;
}

int get(int i) {
  int ans = 0;
  for (; i >= 0; i = (i & (i + 1)) - 1)
    ans += fenw[i];
  return ans;
}

int main() {
#ifdef DEBUG
  freopen("input.txt", "r", stdin);
  freopen("output.txt", "w", stdout);
#else
  ios::sync_with_stdio(false);
  cin.tie(0);
  cout.tie(0);
#endif

  lg[0] = -1;
  for (int i = 1; i < MAXN; ++i)
    lg[i] = lg[i / 2] + 1;

  int n;
  cin >> n;
  vector<int> A(n);
  vector<int> id(n);
  for (int i = 0; i < n; ++i) {
    cin >> A[i];
    --A[i];
    id[A[i]] = i;
  }
  for (int i = 0; i < n; ++i)
    table[i][0] = A[i];
  for (int j = 1; j < K; ++j) {
    for (int i = 0; i + (1 << j) <= n; ++i)
      table[i][j] = min(table[i][j - 1], table[i + (1 << (j - 1))][j - 1]);
  }
  int q;
  cin >> q;
  vector<pair<int, pair<int, int>>> events(2 * q);
  vector<pair<int, int>> rows(q);
  for (int i = 0; i < q; ++i) {
    int r1, c1, r2, c2;
    cin >> r1 >> c1 >> r2 >> c2;
    events[2 * i + 0] = make_pair(i, make_pair(r1, c1));
    events[2 * i + 1] = make_pair(i, make_pair(r2, c2));
    rows[i] = make_pair(min(r1, r2), i);
  }
  sort(events.begin(), events.end(), [](pair<int, pair<int, int>> &a, pair<int, pair<int, int>> &b) {
    return a.second.first < b.second.first;
  });
  vector<int> val(q, MAXN);
  int j = 0;
  for (auto event : events) {
    int r = event.second.first, c = event.second.second;
    while (j < r) {
      add(0, 0, n, id[j], 1);
      ++j;
    }
    int i;
    if (r == c) {
      i = 0;
    } else {
      i = get_at_least(0, 0, n, r - c);
    }
    if (val[event.first] == MAXN) {
      val[event.first] = i;
    } else {
      int i2 = val[event.first];
      rows[event.first].first = min(rows[event.first].first, get(min(i, i2), max(i, i2)));
    }
  }
  sort(rows.begin(), rows.end());
  vector<pair<int, int>> ans(q);
  j = 0;
  for (auto t : rows) {
    while (j < t.first) {
      add(id[j], 1);
      ++j;
    }
    ans[t.second] = make_pair(t.first, t.first - get(val[t.second] - 1));
  }
  for (auto t : ans)
    cout << t.first << ' ' << t.second << '\n';
  return 0;
}
