#include <iostream>
#include <vector>
#include <algorithm>
#include <set>
#include <tuple>

using namespace std;

using ll = long long;
template<typename T>
using v = vector<T>;
template <typename T1, typename T2>
using pa = pair<T1, T2>;

template<typename T>
istream& operator>>(istream& in, v<T>& a) {
	for (T& x : a) {
		in >> x;
	}
	return in;
}

struct segment_tree {
	ll n;
	v<ll> st;
	segment_tree(ll n) : n(n), st(4 * n) {

	}

	ll update(ll p, ll x, ll i = 0, ll L = 0, ll R = -1) {
		if (R == -1) {
			R = n;
		}
		if (R <= p || L > p) {
			return st[i];
		}
		else if (R - L == 1) {
			return st[i] = x;
		}
		else {
			ll M = (L + R) >> 1;
			return st[i] = update(p, x, (i << 1) + 1, L, M) + update(p, x, (i << 1) + 2, M, R);
		}
	}

	ll get_mi_pos(ll x, ll i = 0, ll L = 0, ll R = -1) {
		if (R == -1) {
			R = n;
		}
		if (st[i] < x) {
			return R;
		}
		else if (R - L == 1) {
			return L;
		}
		else {
			ll M = (L + R) >> 1;
			ll a = get_mi_pos(x, (i << 1) + 1, L, M);
			if (a == M) {
				return get_mi_pos(x - st[(i << 1) + 1], (i << 1) + 2, M, R);
			}
			else {
				return a;
			}
		}
	}

	ll get_range(ll inL, ll inR, ll i = 0, ll L = 0, ll R = -1) {
		if (R == -1) {
			R = n;
		}
		if (inR <= L || inL >= R || R - L == 0) {
			return 0;
		}
		else if (L >= inL && R <= inR) {
			return st[i];
		}
		else {
			ll M = (L + R) >> 1;
			return get_range(inL, inR, (i << 1) + 1, L, M) + get_range(inL, inR, (i << 1) + 2, M, R);
		}
	}
};

struct sparse_table {
	ll n;
	v<ll> log;
	v<v<pa<ll, ll>>> st;
	sparse_table(const v<ll>& a) : n(a.size()), log(a.size() + 1) {
		for (ll i = 2; i <= n; ++i) {
			log[i] = log[i >> 1] + 1;
		}
		st = v<v<pa<ll, ll>>>(log[n] + 1, v<pa<ll, ll>>(n));
		for (ll i = 0; i < n; ++i) {
			st[0][i] = { a[i], i };
		}
		for (ll j = 1; j <= log[n]; ++j) {
			for (ll i = 0; i <= (n - (1ll << j)); ++i) {
				st[j][i] = min(st[j - 1][i], st[j - 1][i + (1ll << (j - 1))]);
			}
		}
	}
	pa<ll, ll> get_range(ll L, ll R) {
		ll j = log[R - L];
		return min(st[j][L], st[j][R - (1ll << j)]);
	}
};

void solve() {
	ll n;
	cin >> n;
	v<ll> a(n);
	cin >> a;
	for (ll& x : a) {
		--x;
	}
	v<ll> inva(n);
	for (ll i = 0; i < n; ++i) {
		inva[a[i]] = i;
	}
	ll q;
	cin >> q;
	v<tuple<ll, ll, ll, ll>> req(q);
	v<tuple<ll, ll, ll>> off_req(q << 1);
	v<ll> off_req_res(q << 1);
	for (ll i = 0; i < q; ++i) {
		ll r1, c1, r2, c2;
		cin >> r1 >> c1 >> r2 >> c2;
		c1 = r1 - c1;
		c2 = r2 - c2;
		req[i] = { r1, c1, r2, c2 };
		off_req[i << 1] = { r1, c1, i << 1 };
		off_req[(i << 1) + 1] = { r2, c2, (i << 1) + 1 };
	}
	sort(off_req.begin(), off_req.end());
	segment_tree st(n);
	ll cp = 0;
	for (auto [r, c, i] : off_req) {
		while (cp < n && cp < r) {
			st.update(inva[cp], 1);
			++cp;
		}
		ll mi_t = st.get_mi_pos(c);
		if (c != 0) {
			++mi_t;
		}
		off_req_res[i] = mi_t;
	}
	v<tuple<ll, ll, ll>> off_req_2;
	v<pa<ll, ll>> req_res(q);
	sparse_table a_st(a);
	for (ll i = 0; i < q; ++i) {
		ll t1 = off_req_res[i << 1];
		ll t2 = off_req_res[(i << 1) + 1];
		if (t1 > t2) {
			swap(t1, t2);
		}
		ll mi = 1e18;
		ll mij = -1;
		if (t1 != t2) {
			auto [smi, smij] = a_st.get_range(t1, t2);
			mi = smi;
			mij = smij;
		}
		/*for (ll j = t1; j < t2; ++j) {
			if (a[j] < mi) {
				mi = a[j];
				mij = j;
			}
		}*/
		if (mi > min(get<0>(req[i]), get<2>(req[i]))) {
			if (get<0>(req[i]) < get<2>(req[i])) {
				req_res[i] = { get<0>(req[i]), get<0>(req[i]) - get<1>(req[i]) };
			}
			else {
				req_res[i] = { get<2>(req[i]), get<2>(req[i]) - get<3>(req[i]) };
			}
		}
		else {
			off_req_2.push_back({ mi, mij, i });
			req_res[i].first = mi;
		}
	}
	sort(off_req_2.begin(), off_req_2.end());
	st = segment_tree(n);
	cp = 0;
	for (auto [mi, mij, i] : off_req_2) {
		while (cp < mi) {
			st.update(inva[cp], 1);
			++cp;
		}
		ll ps = st.get_range(0, mij);
		req_res[i].second = req_res[i].first - ps;
	}
	for (auto [rr, rc] : req_res) {
		cout << rr << ' ' << rc << '\n';
	}
}

int main() {
	ll t = 1;
	//cin >> t;
	for (ll i = 0; i < t; ++i) {
		solve();
	}
}