#include <iostream>
#include <vector>
#include <algorithm>
#include <map>
#include <random>

using namespace std;

mt19937 twister(1488);

struct Treap {
    int left, right;
    int64_t sum;
    int64_t value;
    int size;

    Treap(int64_t val) {
        value = sum = val;
        size = 1;
        left = right = 0;
    }

    Treap() {

    }
};

using tree_t = int;

const int MAX_MEM = 5e7;

Treap mem[MAX_MEM];
int mem_ptr = 1;

int get_size(tree_t tree) {
    if (tree == 0) return 0;
    return mem[tree].size;
}

int64_t get_sum(tree_t tree) {
    if (tree == 0) return 0;
    return mem[tree].sum;
}

void update(tree_t tree) {
    if (tree == 0);
    mem[tree].size = get_size(mem[tree].left) + get_size(mem[tree].right) + 1;
    mem[tree].sum = get_sum(mem[tree].left) + get_sum(mem[tree].right) + mem[tree].value;
}

tree_t copy(tree_t tree) {
    mem[mem_ptr++] = mem[tree];
    return mem_ptr - 1;
}

pair<tree_t, tree_t> split(tree_t tree, int64_t k) {
    if (tree == 0) return {0, 0};
    if (mem[tree].value < k) {
        auto tmp = split(mem[tree].right, k);
        mem[tree].right = tmp.first;
        update(tree);
        return {tree, tmp.second};
    } else {
        auto tmp = split(mem[tree].left, k);
        mem[tree].left = tmp.second;
        update(tree);
        return {tmp.first, tree};
    }
}

int opers = 0;

struct Query {
    int t;
    int64_t k;
    int i, idx;
    Query(int a, int64_t b, int c, int d) {
        t = a;
        k = b;
        i = c;
        idx = d;
    }
};

tree_t merge(tree_t a, tree_t b) {
    if (a == 0) return b;
    if (b == 0) return a;
    uniform_int_distribution<int> distr(1, get_size(a) + get_size(b));
    int rnd = distr(twister);
    opers++;
    if (rnd < get_size(b)) {
        mem[a].right = merge(mem[a].right, b);
        update(a);
        return a;
    } else {
        mem[b].left = merge(a, mem[b].left);
        update(b);
        return b;
    }
}

tree_t insert(tree_t tree, int64_t x) {
    auto tmp = split(tree, x);
    mem[mem_ptr++] = Treap(x);
    return merge(tmp.first, merge(mem_ptr - 1, tmp.second));
}


int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(0);
    int n, q; cin >> n >> q;
    vector<int64_t> as(n);
    for (int i = 0; i < n; i++) {
        cin >> as[i];
    }
    vector<int64_t> as_copy = as;
    sort(as.begin(), as.end());
    // as.resize(unique(as.begin(), as.end()) - as.begin());

    tree_t current_deltas = 0;
    vector<Query> qs;
    vector<int64_t> answers(q);
    vector<vector<Query>> idx_qs(n);
    for (int j = 0; j < q; j++) {
        int t;
        int64_t k;
        int i;
        cin >> t;
        cin >> k;
        if (t == 3) {
            cin >> i;
            i--;
        }
        qs.emplace_back(t, k, i, j);
        int lo = 0;
        int hi = n;
        if (t == 3) {
        while (hi - lo > 1) {
            int mid = (lo + hi) / 2;
            if (as[mid] <= as_copy[i]) {
                lo = mid;
            } else {
                hi = mid;
            }
        }
        idx_qs[lo].push_back(qs.back());
        }
    }

    for (int i = n - 1; i >= 0; i--) {
        for (auto q : idx_qs[i]) {
            int64_t k = q.k;
            auto tmp = split(current_deltas, k);
            int64_t ans = as[i] + get_sum(tmp.first) + get_size(tmp.second) * 1ll * k;
            answers[q.idx] = ans;
            current_deltas = merge(tmp.first, tmp.second);
        }
        if (i != 0) {
            current_deltas = insert(current_deltas, as[i] - as[i - 1]);
        }
    }
    vector<pair<int64_t, int>> deltas;
    for (int i = 0; i < n - 1; i++) {
        deltas.emplace_back(as[i + 1] - as[i], i + 1);
    }
    sort(deltas.begin(), deltas.end());
    vector<int64_t> deltas_pref_sums(n);
    deltas_pref_sums[0] = 0;
    vector<int64_t> deltas_suff_sums_partial(n);
    deltas_suff_sums_partial[n - 1] = 0;
    for (int i = n - 2; i >= 0; i--) {
        deltas_suff_sums_partial[i] = deltas_suff_sums_partial[i + 1] + deltas[i].second;
    }
    for (int i = 1; i < n; i++) {
        deltas_pref_sums[i] = deltas_pref_sums[i - 1] + (deltas[i - 1].second * 1ll * deltas[i - 1].first);
    }
    for (int j = 0; j < q; j++) {
        Query query = qs[j];
        int t = query.t;
        int64_t k = query.k;
        int i = query.i;
        if (t == 1) {
            int idx = upper_bound(deltas.begin(), deltas.end(), make_pair(k, n + 1488)) - deltas.begin();
            cout << n - idx << '\n';
        } else if (t == 2) {
            int idx = upper_bound(deltas.begin(), deltas.end(), make_pair(k, n + 1488)) - deltas.begin();
            int64_t ans = deltas_pref_sums[idx] + deltas_suff_sums_partial[idx] * 1ll * k;
            cout << ans << '\n';
        } else if (t == 3) {
            cout << answers[j] << '\n';
        }
    }
}
