#include <iostream>
#include <set>
#include <map>
#include <string>
#include <vector>
#include <algorithm>
#include <queue>
#include <cmath>
#include <iomanip>
#include <deque>

using namespace std;
typedef long long ll;
typedef long double ld;
typedef unsigned long long ull;

const ll INF = 1e18;

vector<ll> h;
vector<ll> f;
vector<bool> no;

struct Dsu {
	vector<ll> p;
	vector<ll> k;
	void init(ll n) {
		p.assign(n, 0);
		k.assign(n, 1);
		for (ll i = 0; i < n; i++) p[i] = i;
	}
	ll get(ll v) {
		if (v == p[v]) return v;
		p[v] = get(p[v]);
		return p[v];
	}
	bool unite_(ll a, ll b) {
		a = get(a), b = get(b);
		if (a == b) return 0;
		p[b] = a;
		k[a] += k[b];
		return 1;
	}
};

void dfs(ll v, ll p, const vector<vector<ll>>& g, vector<bool>& used) {
	used[v] = 1;
	h[v] = 0;
	if (p > -1) h[v] = h[p] + 1;
	f[v] = h[v];
	ll x = 0;
	for (ll i : g[v]) {
		if (!used[i]) {
			x++;
			dfs(i, v, g, used);
			f[v] = min(f[v], f[i]);
		}
		else if (i != p) {
			if (p != -1) f[p] = min(f[p], h[i]);
		}
		else {
			x++;
		}
	}
	if (f[v] >= h[v] && x > 1) {
		no[v] = 1;
	}
}

int main() {
	ios_base::sync_with_stdio(0);
	cin.tie(0);
	cout.tie(0);
	ll t;
	cin >> t;
	while (t--) {
		ll n, m;
		cin >> n >> m;
		h.assign(n, 0);
		f.assign(n, 0);
		no.assign(n, 0);
		vector<vector<ll>> g(n);
		vector<ll> ans(n, INF);
		vector<ll> dp(n, INF);
		vector<pair<ll, ll>> p(n, { -1, -1 });
		for (ll i = 0; i < m; i++) {
			ll u, v;
			cin >> u >> v;
			u--; v--;
			g[u].push_back(v);
			g[v].push_back(u);
		}
		vector<bool> used(n, 0);
		dfs(0, -1, g, used);
		vector<Dsu> d(n);
		for (ll i = 0; i < n; i++) {
			if (no[i]) {
				d[i].init(n);
				for (ll j = 0; j < n; j++) {
					for (ll k : g[j]) {
						if (j != i && k != i) {
							d[i].unite_(j, k);
						}
					}
				}
			}
		}
		vector<bool> used1(n, 0), used2(n, 0);
		queue<ll> que1, que2;
		que1.push(0);
		ans[0] = 0;
		ll type = 1;
		while (que1.size() > 0 || que2.size() > 0) {
			if (type == 1) {
				ll v = que1.front();
				que1.pop();
				if (que1.size() == 0) type = 2;
				used1[v] = 1;
				for (ll i : g[v]) {
					if (dp[i] > ans[v] + 1) {
						dp[i] = ans[v] + 1;
						if (p[i].first == -1) p[i].first = v;
						else p[i].second = v;
						if (!used2[i]) que2.push(i);
					}
				}
			}
			else {
				ll v = que2.front();
				que2.pop();
				if (que2.size() == 0) type = 1;
				used2[v] = 1;
				for (ll i : g[v]) {
					bool b = 1;
					if (no[v]) {
						b = (d[v].get(i) == d[v].get(p[v].first));
						if (b == 0 && p[v].second > -1) b = (d[v].get(i) == d[v].get(p[v].first));
					}
					if (ans[i] > dp[v] + 1 && b) {
						ans[i] = dp[v] + 1;
						if (!used1[i]) que1.push(i);
					}
				}
			}
		}
		for (ll i = 1; i < n; i++) {
			if (ans[i] < INF) cout << ans[i] << " ";
			else cout << "-1 ";
		}
		cout << "\n";
	}
	return 0;
}