#include <iostream>
#include <vector>
#include <algorithm>
#include <cmath>
#include <stack>
#include <deque>
#include <queue>
#include <set>
#include <map>
#include <unordered_map>
#include <string>
#include <chrono>
#include <random>
#include <iomanip>

#define all(a) (a).begin(), (a).end()
#define debug(a) std::cerr << #a << ": " << a << '\n';

using ll = long long;
using ld = long double;
using str = std::string;
using pll = std::pair<ll, ll>;

const ll Max = 1e18, Min = -1e18;
const ld Pi = std::acos(-1);
const ld eps = 1e-12;
const ll Mod = 1234567891;

struct Node {
	ll sum, cnt;
	ll sumc, cntc;

	Node operator+ (Node other) {
		return { sum + other.sum, cnt + other.cnt, sumc + other.sumc, cntc + other.cntc };
	}
};

struct Q {
	ll type;
	ll k;
	ll i;
	ll num;
};

struct SegTree {
	std::vector<Node> tree;
	ll s = 1;

	void build(std::vector<ll>& list) {
		while (s < list.size()) {
			s *= 2;
		}
		tree.resize(s * 2 - 1, { 0, 0 });
	}

	pll getm(ll l_find, ll r_find, ll l, ll r, ll i) {
		if (l >= r_find || r <= l_find) return { 0, 0 };
		if (l >= l_find && r <= r_find) return { tree[i].cnt, tree[i].cntc };
		ll mid = l + (r - l) / 2;
		pll first = getm(l_find, r_find, l, mid, i * 2 + 1);
		pll second = getm(l_find, r_find, mid, r, i * 2 + 2);
		return { first.first + second.first, first.second + second.second };
	}

	pll getl(ll l_find, ll r_find, ll l, ll r, ll i) {
		if (l >= r_find || r <= l_find) return { 0, 0 };
		if (l >= l_find && r <= r_find) return { tree[i].sum, tree[i].sumc };
		ll mid = l + (r - l) / 2;
		pll first = getm(l_find, r_find, l, mid, i * 2 + 1);
		pll second = getm(l_find, r_find, mid, r, i * 2 + 2);
		return { first.first + second.first, first.second + second.second };
	}

	Node get(ll k) {
		Node ans;
		pll first = getm(k, s, 0, s, 0);
		pll second = getl(0, k, 0, s, 0);
		ans.cnt = first.first;
		ans.cntc = first.second;
		ans.sum = second.first;
		ans.sumc = second.second;
		return ans;
	}

	void change(ll pos, ll val, ll cnt, ll l, ll r, ll i) {
		if (l > pos || r <= pos) return;
		if (r - l == 1) {
			tree[i].sum += val;
			tree[i].cnt = 1;
			tree[i].cntc = cnt;
			tree[i].sumc += val * cnt;
			return;
		}
		ll mid = l + (r - l) / 2;
		change(pos, val, cnt, l, mid, i * 2 + 1);
		change(pos, val, cnt, mid, r, i * 2 + 2);
		tree[i] = tree[i * 2 + 1] + tree[i * 2 + 2];
	}
};

void solve() {
	ll n, q;
	std::cin >> n >> q;
	std::vector<pll> tmp(n);
	for (int i = 0; i < n; ++i) {
		std::cin >> tmp[i].first;
		tmp[i].second = i;
	}
	std::vector<ll> pos(n);
	std::sort(all(tmp));
	for (int i = 0; i < n; ++i) {
		pos[tmp[i].second] = i;
	}
	std::vector<ll> list(n), copy;
	list[0] = tmp[0].first;
	for (int i = 1; i < n; ++i) {
		list[i] = tmp[i].first - tmp[i - 1].first;
	}
	copy = list;
	std::sort(all(copy));
	copy.resize(std::unique(all(copy)) - copy.begin());
	SegTree tree;
	tree.build(copy);
	std::vector<std::vector<Q>> calc(n);
	std::vector<pll> typ(q);
	std::vector<Node> answers(q);
	for (int i = 0; i < q; ++i) {
		ll t, k, ind;
		std::cin >> t >> k;
		typ[i].first = t;
		typ[i].second = k;
		if (t == 3) {
			std::cin >> ind;
			calc[ind].push_back({ t, k, ind, i});
		}
		else {
			calc[0].push_back({ t, k, 0, i});
		}
	}
	for (int i = n - 1; i >= 0; --i) {
		while (!calc[i].empty()) {
			answers[calc[i].back().num] = tree.get(std::lower_bound(all(copy), calc[i].back().k) - copy.begin());
			calc[i].pop_back();
		}
		tree.change(std::lower_bound(all(copy), list[i]) - copy.begin(), list[i], i, 0, tree.s, 0);
	}
	for (int i = 0; i < q; ++i) {
		if (typ[i].first == 1) {
			std::cout << answers[i].cnt << '\n';
		}
		else if (typ[i].first == 2) {
			std::cout << answers[i].cntc * typ[i].second  + answers[i].sumc - 1 << '\n';
		}
		else {
			std::cout << answers[i].cnt * typ[i].second + answers[i].sum << '\n';
		}
	}
}

int main() {
	std::ios::sync_with_stdio(0);
	std::cin.tie(0);
	std::cout.tie(0);
	std::cout << std::fixed << std::setprecision(100);
	ll t = 1;
	// std::cin >> t;
	while (t--) {
		solve();
	}
}
