#ifndef LOCAL
#pragma GCC optimize("Ofast")
#pragma GCC optimize("O3")
#endif

#include <iostream>
#include <vector>
#include <list>
#include <string>
#include <algorithm>

#define len(a) (int)a.size()

typedef long long ll;


const int INF = (int)1e9;

using namespace std;

void run();

signed main()
{
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
#else
    ios_base::sync_with_stdio(0);
    cin.tie(0);
    cout.tie(0);
#endif

    run();

}

const char TYPE_UNITE = 1;
const char TYPE_FIRST = 2;
const char TYPE_SECOND = 3;
const char TYPE_THIRD = 4;

struct Event
{
    ll time;
    char type;
    int request_id;

    Event() : Event(-1, 127) {}
    Event(ll time, char type) : time(time), type(type) {}

    bool operator<(const Event& other) const
    {
        return tie(time, type) < tie(other.time, other.type);
    }
};

void run()
{
    int n, q;
    cin >> n >> q;

    vector<ll> arr(n);
    for (int i = 0; i < n; ++i)
    {
        cin >> arr[i];
    }

    sort(arr.begin(), arr.end());
    vector< pair<ll, int> > groups;
    groups.reserve(n);
    ll last_group = arr[0];
    int last_count = 1;
    for (int i = 1; i < n; ++i)
    {
       if (arr[i] != last_group)
       {
           groups.emplace_back(last_group, last_count);
           last_group = arr[i];
           last_count = 1;
       }
       else
       {
           ++last_count;
       }
    }
    groups.emplace_back(last_group, last_count);

    vector<Event> events;
    events.reserve(len(groups) + q);
    for (int i = 0; i < q; ++i)
    {
        int type;
        ll time;
        cin >> type >> time;
        if (type == 1)
        {
            events.emplace_back(time, TYPE_FIRST);
            events.back().request_id = i;
        }
        else if (type == 2)
        {
            events.emplace_back(time, TYPE_SECOND);
            events.back().request_id = i;
        }
        else
        {
            cout << "PIZDA\n";
        }
    }
    for (int i = 0; i < len(groups) - 1; ++i)
    {
        events.emplace_back(groups[i + 1].first - groups[i].first, TYPE_UNITE);
        events.back().request_id = i;
    }
    sort(events.begin(), events.end());

    vector<ll> prefix_sums(len(groups));
    prefix_sums[0] = groups[0].second;
    for (int i = 1; i < len(groups); ++i)
    {
        prefix_sums[i] = prefix_sums[i - 1] + groups[i].second;
    }

    ll cur_raise = 0;
    for (int i = 0; i < len(groups); ++i)
    {
        cur_raise += (ll)(len(groups) - i - 1) * groups[i].second;
    }

    int cur_groups = len(groups);
    vector<ll> ans(q);
    ll cur_sum = 0;
    ll last_time = 0;
    for (Event& event : events)
    {
        cur_sum += (event.time - last_time) * cur_raise;
        if (event.type == TYPE_UNITE)
        {
            --cur_groups;
            cur_raise -= prefix_sums[event.request_id];
        }
        else if (event.type == TYPE_FIRST)
        {
            ans[event.request_id] = cur_groups;
        }
        else if (event.type == TYPE_SECOND)
        {
            ans[event.request_id] = cur_sum;
        }
        last_time = event.time;
    }

    for (ll i : ans)
    {
        cout << i << '\n';
    }
}
