#define _GLIBCXX_DEBUG
#include <climits>
#include <iostream>
#include <queue>
#include <algorithm>
#include <map>
#include <set>
#include <string>
#include <vector>

struct dsu
{
	std::vector<int> p;
	std::vector<int> rank;
	std::vector<std::pair<int,int>> undo;
	int get(int at)
	{
		if(at == p[at]) return at;
		return get(p[at]);
	}
	void merge(int u, int v)
	{
		//printf("merge %d, %d\n", u, v);
		u = get(u);
		v = get(v);
		if(rank[u] < rank[v]) std::swap(u, v);
		if(rank[u] == rank[v])
		{
			rank[u]++;
			undo.push_back({v, ~p[v]}); // negated -> also decrement rank of new value
			p[v] = u;
		}
		else
		{
			undo.push_back({v, p[v]});
			p[v] = u;
		}
	}
	void pop()
	{
		auto[v,ov] = undo.back();
		//printf("pop %d, %d\n", p[v], v);
		//for(int x : d.p) printf("%d ", x); printf("\n");
		//for(int x : d.rank) printf("%d ", x); printf("\n");
		undo.pop_back();
		if(ov < 0) rank[p[v]]--, ov = ~ov;
		p[v] = ov;
	}
};
struct query
{
	int u, v;
	char type; // 0 -> merge, 1 -> get
};
std::vector<int> qres;
	std::vector<std::pair<int,int>> edges;
struct stree
{
	std::vector<query> own_queries;
	stree*l, *r;
	void mkl() { if(!l) l = new stree { {}, nullptr, nullptr }; }
	void mkr() { if(!r) r = new stree { {}, nullptr, nullptr }; }
	void merge(int u, int v, int ql, int qr, int L, int R)
	{
		if(ql <= L && R <= qr)
		{
			own_queries.push_back({u, v, 0});
			return;
		}
		if(L >= qr || R <= ql) return;
		mkl();
		l->merge(u, v, ql, qr, L, (L+R)/2);
		mkr();
		r->merge(u, v, ql, qr, (L+R)/2, R);
	}
	void get(int u, int qi, int at, int L, int R)
	{
		int M = (L+R)/2;
		if(M == L)
		{
			own_queries.push_back({u, qi, 1});
			return;
		}
		if(at < M)
		{
			mkl(); l->get(u, qi, at, L, M);
		}
		else
		{
			mkr(); r->get(u, qi, at, M, R);
		}
	}
	void run(dsu& ds)
	{
		int merges = 0;
		for(auto[u, v, t] : own_queries)
		{
			if(t == 0)
			{
				merges++;
				ds.merge(u, v);
				edges.push_back({u, v});
			}
			else
			{
				//printf("getting %d\n", u);
				//printf("current edges: ");
				//for(auto[u, v] : edges) printf("%d-%d ", u, v); 
				//printf("\n");
				//printf("dsu: "); for(int x : ds.p) printf("%d ", x); printf("\n");
				qres[v] = ds.get(u);
				//printf("ret=%d (to %d)\n", qres[v], v);
			}
		}
		if(l) l->run(ds);
		if(r) r->run(ds);
		while(merges --> 0)
		{
			ds.pop();
			edges.pop_back();
		}
	}
};
int main()
{
	// transition search: iterate over vertex, remove it, find what neighbors are in the same component
	// offline dcp
	// >=2 in the same component -> connect to common new vertex
	// e.g. standing at 0, 1 adjacent, 2 adjacent to 1
	// remove vertex 1
	// 0 and 2 are now in different components :(
	// remove vertex -> actually temporarily add it twice
	int T; std::cin >> T;
	while(T --> 0)
	{
		int n, m; scanf("%d%d", &n, &m);
		qres = {};
		edges = {};
		stree t {};
		int ptwo = n;
		while(ptwo & ptwo-1) ptwo++;
		std::vector<std::vector<int>> graph(n);
		for(int ei = 0; ei < m; ei++)
		{
			int u,v; scanf("%d%d", &u, &v); u--; v--;
			if(u > v) std::swap(u, v);
			// add edge from [0, u)
			// add edge from [u+1, v)
			// add edge from [v+1, n)
			if(0 != u) t.merge(u, v, 0, u, 0, ptwo);
			if(u+1 != v) t.merge(u, v, u+1, v, 0, ptwo);
			if(v+1 != n) t.merge(u, v, v+1, n, 0, ptwo);
			graph[u].push_back(v);
			graph[v].push_back(u);
		}

		for(int i = 0; i < n; i++)
		{
			for(int to : graph[i])
			{
				qres.push_back(0);
				//printf("%d->%d: qres.size=%d\n", i, to, qres.size());
				t.get(to, qres.size()-1, i, 0, ptwo);
			}
		}
		dsu d;
		d.p.resize(n);
		d.rank.resize(n);
		for(int i = 0; i < n; i++) d.p[i] = i;
		for(int i = 0; i < n; i++) d.rank[i] = 0;
		t.run(d);
		//for(int x : qres) printf("%d ", x); printf("\n");
		int qi = 0;
		std::vector<std::vector<int>> g2(n);
		for(int i = 0; i < n; i++)
		{
			//printf("adjacent to %d:\n", i);
			std::map<int, std::vector<int>> vc;
			for(int to : graph[i])
			{
				//printf("no %d, %d from %d: %d\n", i, to, qi, qres[qi]);
				//printf("qre.size=%d\n", qres.size());
				int qans = qres[qi++];
				vc[qans].push_back(to);
			}
			for(auto[_, vs] : vc)
			{
				if(vs.size() < 2) continue;
				g2.push_back({});
				int ni = g2.size() - 1;
				//printf("%d: ", _);
				for(int u : vs)
				{
					//printf("%d ", u);
					g2[u].push_back(ni);
					g2[ni].push_back(u);
				}
				//printf("\n");
			}
		}
		std::vector<int> dist(g2.size(), INT_MAX);
		dist[0] = 0;
		std::queue<int> q;
		q.push(0);
		while(q.size())
		{
			int at = q.front(); q.pop();
			int cd = dist[at];
			for(int to : g2[at])
			{
				int nd = cd + 1;
				if(dist[to] <= nd) continue;
				dist[to] = nd;
				q.push(to);
			}
		}
		for(int i = 1; i < n; i++)
			printf("%d ", dist[i] == INT_MAX ? -1 : dist[i]);
		//for(int x : dist) printf("%d ", x); printf("\n");
		printf("\n");
	}
}
