#include <numeric>
#include <cassert>
#include <queue>
#include <sstream>
#include <functional>
#include <random>
#include <ctime>
#include <iostream>
#include <vector>
#include <set>
#include <algorithm>

using namespace std;
mt19937 rnd(time(0) + 239);

 #define int long long
constexpr int inf = 1e9 + 7;
using ll = long long;


struct Query {
    int t, k, i, res_i;
};

struct FenwickTree {
    vector<int> tree;

    FenwickTree(int n): tree(n, 0) {}

    int Get(int right) {
        int res = 0;
        while (right >= 0) {
            res += tree[right];
            right = (right & (right + 1)) - 1;
        }
        return res;
    }

    void Update(int right, int delta) {
        while (right < tree.size()) {
            tree[right] += delta;
            right |= right + 1;
        }
    }

    void UpdateRange(int right, int delta) {
        Update(0, delta);
        if (right + 1 < tree.size()) {
            Update(right + 1, -delta);
        }
    }
};


void Solve(istream &in, ostream &out) {
    int n, q;
    in >> n >> q;
    vector<ll> C(n);
    for (ll &c_i : C) {
        in >> c_i;
    }
    vector<int> order(n);
    iota(order.begin(), order.end(), 0);
    sort(order.begin(), order.end(), [&C](int i, int j) { return C[i] < C[j]; });

    vector<Query> Q;
    vector<int> P(n);
    for (int i = 0; i < n; ++i) {
        P[order[i]] = i;
        if (i != 0) {
            Q.push_back({0, C[order[i]] - C[order[i - 1]], i - 1, -1});
        }
    }
    for (int i = 0; i < q; ++i) {
        Query query;
        in >> query.t >> query.k;
        if (query.t == 3) {
            in >> query.i;
            --query.i;
        }
        query.res_i = i;
        Q.push_back(query);
    }
    sort(Q.begin(), Q.end(), [](Query lhs, Query rhs) {
        return make_pair(lhs.k, lhs.t) < make_pair(rhs.k, rhs.t);
    });

    FenwickTree tree(n), dead_tree(n);
    vector<int> result(q);
    int last = 0, dead = 0;
    int s_delta = 0, growth = (n - 1) * n / 2;
    for (auto [t, k, i, res_i] : Q) {
        int d_k = k - last;
        s_delta += growth * d_k;
        if (t == 0) {
            tree.UpdateRange(i, k);
            dead_tree.UpdateRange(i, 1);
            growth -= i + 1;
            ++dead;
        } else if (t == 1) {
            result[res_i] = n - dead;
        } else if (t == 2) {
            result[res_i] = s_delta;
        } else if (t == 3) {
            i = P[i];
            int dead_val = dead_tree.Get(i);
            result[res_i] = k * (n - i - 1 - dead_val) + tree.Get(i) + C[order[i]];
        } else {
            exit(1);
        }
        last += d_k;
    }

    for (int res_i : result) {
        out << res_i << '\n';
    }
}


signed main() {
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
#else
    ios_base::sync_with_stdio(false);
    cin.tie(0);
    cout.tie(0);
#endif
    Solve(cin, cout);
//    ostringstream test;
//    GenFull(test);
//    istringstream in(test.str());
//    ostringstream out;
//    Solve(in, cout);
    cerr << clock() * 1.0 / CLOCKS_PER_SEC << '\n';
    return 0;
}
