#include <bits/stdc++.h>

#define int long long

using namespace std;

const int INF = 1e15;

struct Edge
{
    int v, u;
    int cap;

    Edge(int v=0, int u=0, int cap=0): v(v), u(u), cap(cap) {}
};

struct Arm
{
    int left, right;
    int cnt;
    int type;

    Arm(int left=0, int right=0, int cnt=0, int type=0): left(left), right(right), cnt(cnt), type(type) {}

    friend istream& operator>>(istream& in, Arm& A) {
        in >> A.left >> A.right >> A.cnt >> A.type;
        A.left -= 1; A.right -= 1;
        return in;
    }
};

int n, m;
vector <Edge> edges;
vector <vector <int>> indexes;
vector <int> first_inf;
vector <int> height;

void add_edge(int a, int b, int c)
{
    edges.push_back({a, b, c});
    indexes[a].push_back(edges.size() - 1);
    edges.push_back({b, a, 0});
    indexes[b].push_back(edges.size() - 1);
}

int add_flow(int v, int cur_flow)
{
    //cout << "! " << v << " " << cur_flow << "\n";
    if (v == n + m + 1)
        return cur_flow;

    vector <int> check;
    if (n <= v && v < n + m) {
        check.push_back(indexes[v][0]);
        for (int i = first_inf[v - n]; i < indexes[v].size(); ++i)
            check.push_back(indexes[v][i]);
    }
    else {
        check = indexes[v];
    }
    for (auto idx: check) {
        auto& e = edges[idx];
        auto u = e.u, cap = e.cap;
        if (cap > 0 && height[u] == height[v] + 1) {
            int new_flow = add_flow(u, min(cur_flow, cap));
            if (new_flow > 0) {
                edges[idx].cap -= new_flow;
                edges[idx ^ 1].cap += new_flow;
                return new_flow;
            }
        }
    }
    return 0;
}

signed main()
{
    int Q; cin >> Q;
    while (Q--) {
        edges.clear();
        indexes.clear();
        height.clear();
        first_inf.clear();

        cin >> n >> m;
        vector <int> arr(n);
        for (auto& i: arr)
            cin >> i;
        vector <Arm> arms(m);
        for (auto& i: arms)
            cin >> i;

        vector <vector <int>> lefts(n), rights(n);
        for (int i = 0; i < m; ++i) {
            lefts[arms[i].left].push_back(i);
            rights[arms[i].right].push_back(i);
        }

        int flow;
        for (int q = 0; q < n; ++q) {
            if (q == 0) {
                flow = 0;
                indexes.assign(n + m + 2, {});
                for (int i = 0; i < n; ++i) {
                    add_edge(i, n + m + 1, arr[i]);
                }
                first_inf.assign(m, 1);
                for (int i = 0; i < m; ++i) {
                    add_edge(n + m, n + i, arms[i].cnt);
                    int left = arms[i].left, right = arms[i].right;
                    if (arms[i].type == 1) {
                        left = 0;
                    }
                    for (int j = left; j <= right; ++j) {
                        add_edge(n + i, j, INF);
                    }
                }
            }
            else {
                for (int i = 0; i < m; ++i) {
                    int left = arms[i].left, right = arms[i].right, type = arms[i].type;
                    if (type == 0)
                        continue;
                    if (left >= q) {
                        int fi = indexes[n + i][first_inf[i]];
                        int cur = edges[fi ^ 1].cap;
                        //cout << "cur " << cur << "\n";
                        flow -= cur;
                        edges[fi].cap = edges[fi ^ 1].cap = 0;
                        first_inf[i] += 1;

                        int idx = indexes[n + m][i];
                        edges[idx].cap += cur;
                        edges[idx ^ 1].cap -= cur;

                        idx = indexes[q - 1][0];
                        edges[idx].cap += cur;
                        edges[idx ^ 1].cap -= cur;
                    }
                    if (right < q) {
                        add_edge(n + i, q, INF);
                    }
                }
            }
/*
            cout << "\n\nq " << q << "\n";
            for (auto e: edges) {
                cout << e.v << " " << e.u << " " << e.cap << "\n";
            }*/

            while (true) {
                height.assign(n + m + 2, -1); height[n + m] = 0;
                deque <int> q = {n + m};
                while (!q.empty()) {
                    int v = q.front(); q.pop_front();
                    vector <int> check;
                    if (n <= v && v < n + m) {
                        check.push_back(indexes[v][0]);
                        for (int i = first_inf[v - n]; i < indexes[v].size(); ++i)
                            check.push_back(indexes[v][i]);
                    }
                    else {
                        check = indexes[v];
                    }
                    for (auto idx: check) {
                        auto e = edges[idx];
                        int u = e.u, cap = e.cap;
                        if (cap > 0 && height[u] == -1) {
                            height[u] = height[v] + 1;
                            q.push_back(u);
                        }
                    }
                }
                if (height[n + m + 1] == -1)
                    break;

                while (true) {
                    int new_flow = add_flow(n + m, INF);
                    if (new_flow == 0)
                        break;
                    flow += new_flow;
                }
            }

            cout << flow << " ";
        }
        cout << "\n";
    }
}
