#include <iostream>
#include <algorithm>
#include <map>
#include <random>
#include <bitset>

struct DSU {
    int32_t* parents;

    void init(int32_t n) {
        parents = new int32_t[n];
        for(int32_t i = 0; i < n; i++)
            parents[i] = i;
    }

    int32_t get_root(int32_t v) {
        if(parents[v] == v)
            return v;
        return parents[v] = get_root(parents[v]);
    }

    bool same(int32_t v1, int32_t v2) {
        return get_root(v1) == get_root(v2);
    }

    void join(int32_t v1, int32_t v2) {
        v1 = get_root(v1);
        v2 = get_root(v2);
        if(v1 == v2)
            return;
        parents[v1] = v2;
    }
};

int main() {
    int32_t num_tests;
    std::cin >> num_tests;

    for(int32_t t = 0 ;t < num_tests; t++) {
        int32_t n, m;
        std::cin >> n >> m;

        std::vector<int32_t>* graph = new std::vector<int32_t>[n];
        for(int32_t i = 0; i < m; i++) {
            int32_t v1, v2;
            std::cin >> v1 >> v2;
            v1--;
            v2--;

            graph[v1].push_back(v2);
            graph[v2].push_back(v1);
        }

        std::bitset<1000>* neighbors = new std::bitset<1000>[n];
        for(int32_t i = 0; i < n; i++)
            for(int32_t j = 0; j < graph[i].size(); j++)
                neighbors[i][graph[i][j]] = true;

        DSU* dsus = new DSU[n];
        for(int32_t i = 0; i < n; i++) {
            dsus[i].init(n);
            for(int32_t j = 0; j < n; j++)
                for(int32_t k = 0; k < graph[j].size(); k++)
                    if(j != i && graph[j][k] != i)
                        dsus[i].join(j, graph[j][k]);
        }


        int32_t* min_from = new int32_t[n];
        bool* used = new bool[n];
        for(int32_t i = 0; i < n; i++) {
            used[i] = false;
            min_from[i] = INT32_MAX;
        }
        min_from[0] = 0;
        for(int32_t z = 0; z < n; z++) {
            int32_t v = -1;
            for(int32_t i = 0; i < n; i++)
                if(!used[i] && (v == -1 || min_from[i] < min_from[v]))
                    v = i;
            if(v == -1 || min_from[v] == INT32_MAX)
                break;
            used[v] = true;

            for(int32_t i = 0; i < n; i++) {
                std::bitset<1000> common = (neighbors[i] & neighbors[v]);
                int32_t common_neighb = common.count();

                if(common_neighb >= 1) {
                    int32_t deleted = common._Find_first();
                    if(dsus[deleted].same(v, i))
                        min_from[i] = std::min(min_from[i], min_from[v] + 2);
                }
            }
        }

        for(int32_t i = 1; i < n; i++) {
            std::cout << (min_from[i] == INT32_MAX ? -1 : min_from[i]) << " ";
        }
        std::cout << "\n";
    }
    return 0;
}