#include <vector>
#include <iomanip>
#include <iostream>
#include <algorithm>

using namespace std;

const long long BASE = 1e6;
const int BASE_SZ = 6;
const int LEN = 6;
struct BigInt{
  bool sign;
  vector<long long> val;
  BigInt() {
    val.resize(LEN, 0);
    sign = true;
  }
  BigInt(long long x) {
    if (x < 0) {
      sign = false;
      x *= -1;
    } else {
      sign = true;
    }
    val.resize(LEN, 0);
    int id = 0;
    while (x) {
      val[id] = x % BASE;
      x /= BASE;
      ++id;
    }
  }
  BigInt(const vector<long long> &_val, bool _sign) {
    val = _val;
    sign = _sign;
  }
};

void add(const vector<long long> &a, const vector<long long> &b, vector<long long> &ans) {
  ans.resize(LEN);
  for (int i = 0; i < LEN; ++i)
    ans[i] = a[i] + b[i];
  for (int i = 0; i + 1 < LEN; ++i) {
    ans[i + 1] += ans[i] / BASE;
    ans[i] %= BASE;
  }
}

void sub(const vector<long long> &a, const vector<long long> &b, vector<long long> &ans) {
  ans.resize(LEN);
  for (int i = 0; i < LEN; ++i)
    ans[i] = a[i] - b[i];
  for (int i = 0; i + 1 < LEN; ++i) {
    if (ans[i] < 0) {
      ans[i] += BASE;
      --ans[i + 1];
    }
  }
}

bool le(const vector<long long> &a, const vector<long long> &b) {
  for (int i = LEN - 1; i >= 0; --i) {
    if (a[i] < b[i])
      return true;
    if (a[i] > b[i])
      return false;
  }
  return false;
}

void add(const BigInt &a, const BigInt &b, BigInt &c) {
  if (a.sign == b.sign) {
    add(a.val, b.val, c.val);
    c.sign = a.sign;
    return;
  }
  if (le(a.val, b.val)) {
    sub(b.val, a.val, c.val);
    c.sign = b.sign;
  } else {
    sub(a.val, b.val, c.val);
    c.sign = a.sign;
  }
}

bool operator <(const BigInt &a, const BigInt &b) {
  if (a.sign && !b.sign)
    return false;
  if (!a.sign && b.sign)
    return true;
  if (a.sign)
    return le(a.val, b.val);
  return le(b.val, a.val);
}

void ten_power(int k, int c, BigInt &to) {
  fill(to.val.begin(), to.val.end(), 0);
  long long pw = c;
  for (int i = 0; i < k % BASE_SZ; ++i)
    pw *= 10;
  to.val[k / BASE_SZ] = pw;
}

const int K = 21;
BigInt dp[(1 << K) + 228];
int pr[(1 << K) + 228];


int main() {
#ifdef DEBUG
  freopen("input.txt", "r", stdin);
  freopen("output.txt", "w", stdout);
#else
  ios::sync_with_stdio(false);
  cin.tie(0);
  cout.tie(0);
#endif

  string s;
  cin >> s;
  long long y;
  cin >> y;
  int n = s.size();
  vector<int> A(n);
  int j = max(0, n - K);
  for (int i = 0; i < n; ++i)
    A[i] = s[i] - '0';
  sort(A.begin(), A.begin() + j, [](int a, int b) {
    return a > b;
  });
  for (int l = j; l < n; ++l) {
    if (j && A[l] > A[j - 1]) {
      swap(A[l], A[j - 1]);
      for (int t = j - 1; t > 0; --t) {
        if (A[t] > A[t - 1])
          swap(A[t], A[t - 1]);
      }
    }
  }
  dp[0] = 0;
  BigInt tmp1, tmp2;
  for (int mask = 1; mask < (1 << (n - j)); ++mask) {
    ten_power(LEN * BASE_SZ - 1, 1, dp[mask]);
    dp[mask].sign = false;
    int sz = __builtin_popcount(mask);
    int swap_cnt = 0;
    for (int i = 0; i < (n - j); ++i) {
      if (!(mask & (1 << i))) {
        ++swap_cnt;
        continue;
      }
      ten_power(n - j - sz, A[j + 1], tmp2);
      add(dp[mask ^ (1 << i)], tmp2, tmp1);
      add(tmp1, BigInt(-y * swap_cnt), tmp2);
      if (dp[mask] < tmp2) {
        dp[mask] = tmp2;
        pr[mask] = i;
      }
    }
  }
  vector<int> B = A;
  int cur = (1 << (n - j)) - 1;
  while (cur) {
    int id = pr[cur];
    cur = cur ^ (1 << id);
    B[j + __builtin_popcount(cur)] = A[j + id];
  }
  for (int x : B)
    cout << x;
  cout << '\n';
  return 0;
}