#include <iostream>
#include <string>
#include <algorithm>
#include <vector>
#include <map>
#include <tuple>
#include <numeric>

using namespace std;

using ll = long long;
template <typename T>
using v = vector<T>;

template <typename T>
istream& operator>> (istream& in, v<T>& a) {
	for (T& x : a) {
		in >> x;
	}
	return in;
}

struct tree_el {
	ll el, p;
	tree_el() : el(0), p(0) {};
	tree_el operator+(const tree_el& other) {
		tree_el res;
		res.el = el + other.el;
		res.p = p + other.p;
		return res;
	}
};

struct push_el {
	ll deltap;
	ll deltael_p;
	ll deltael;
	push_el() : deltap(0), deltael_p(0), deltael(0) {}
	tree_el mul(const tree_el& other, ll sz) {
		tree_el res;
		res.el = other.el + sz * deltael + deltael_p * other.p;
		res.p = other.p + sz * deltap;
		return res;
	}
};
push_el operator*(const push_el& a, const push_el& b) {
	push_el res;
	res.deltael = a.deltael + b.deltael + a.deltael_p * b.deltap;
	res.deltael_p = a.deltael_p + b.deltael_p;
	res.deltap = a.deltap + b.deltap;
	return res;
}

struct segment_tree {
	ll n;
	v<tree_el> st;
	v<push_el> p;
	tree_el neutral;
	push_el p_neutral;

	segment_tree() = default;

	segment_tree(const v<tree_el>& a) : n(a.size()), st(4 * a.size()), neutral(), p_neutral() {
		p = v<push_el>(4 * a.size(), p_neutral);
		build(a, 0, 0, n);
	}

	tree_el combine_els(tree_el a, tree_el b) {
		return a + b;
	}

	tree_el build(const v<tree_el>& a, ll i, ll L, ll R) {
		if (R - L != 0) {
			if (R - L == 1) {
				return st[i] = a[L];
			}
			else {
				ll M = (L + R) >> 1;
				return st[i] = combine_els(build(a, (i << 1) + 1, L, M), build(a, (i << 1) + 2, M, R));
			}
		}
	}

	void push(ll i, ll sz) {
		st[i] = p[i].mul(st[i], sz);
		ll a = (i << 1) + 1;
		if (a < p.size()) {
			p[a] = p[i] * p[a];
			ll b = (i << 1) + 2;
			if (b < p.size()) {
				p[b] = p[i] * p[b];
			}
		}
		p[i] = p_neutral;
	}

	tree_el get_range(ll inL, ll inR, ll i = 0, ll L = 0, ll R = -1) {
		if (R == -1) {
			R = n;
		}
		if (R - L != 0) {
			push(i, R - L);
			if (R <= inL || L >= inR) {
				return neutral;
			}
			else if (L >= inL && R <= inR) {
				return st[i];
			}
			else {
				ll M = (L + R) >> 1;
				return combine_els(get_range(inL, inR, (i << 1) + 1, L, M), get_range(inL, inR, (i << 1) + 2, M, R));
			}
		}
	}

	tree_el mul_range(ll inL, ll inR, push_el inX, ll i = 0, ll L = 0, ll R = -1) {
		if (R == -1) {
			R = n;
		}
		if (R - L != 0) {
			push(i, R - L);
			if (R <= inL || L >= inR) {
				return st[i];
			}
			else if (L >= inL && R <= inR) {
				p[i] = inX * p[i];
				push(i, R - L);
				return st[i];
			}
			else {
				ll M = (L + R) >> 1;
				return st[i] = combine_els(mul_range(inL, inR, inX, (i << 1) + 1, L, M), mul_range(inL, inR, inX, (i << 1) + 2, M, R));
			}
		}
	}
};

int main() {
	ll n, q;
	cin >> n >> q;
	v<ll> c(n);
	cin >> c;
	segment_tree st;
	v<ll> invs;
	v<ll> aa;
	v<v<ll>> a_c;
	{
		v<pair<ll, ll>> cc(c.size());
		for (ll i = 0; i < c.size(); ++i) {
			cc[i] = { c[i], i };
		}
		sort(cc.begin(), cc.end(), greater<pair<ll, ll>>());
		invs = v<ll>(c.size());
		for (ll i = 0; i < c.size(); ++i) {
			invs[cc[i].second] = i;
		}
		map<ll, v<ll>> a;
		for (ll i = 0; i < cc.size() - 1; ++i) {
			if (cc[i].first != cc[i + 1].first) {
				a[cc[i].first - cc[i + 1].first].push_back(i + 1);
			}
		}
		v<ll> ps(cc.size());
		{
			ll p = 0;
			for (ll i = 0; i < n; ) {
				ll j = i;
				while (j < n && cc[i].first == cc[j].first) {
					++j;
				}
				for (ll k = i; k < j; ++k) {
					ps[k] = p;
				}
				i = j;
				++p;
			}
		}
		v<tree_el> prest(c.size());
		for (ll i = 0; i < c.size(); ++i) {
			prest[i].el = cc[i].first;
			prest[i].p = ps[i];
		}
		st = segment_tree(prest);
		a[0] = v<ll>();
		aa = v<ll>(a.size() - 1);
		a_c = v<v<ll>>(a.size() - 1);
		v<pair<ll, v<ll>>> a2(a.begin(), a.end());
		for (ll i = 0; i < (ll)a.size() - 1; ++i) {
			aa[i] = a2[i + 1].first - a2[i].first;
			a_c[i] = a2[i + 1].second;
		}
	}
	v<tuple<ll, ll, ll, ll>> reqs(q);
	for (ll req = 0; req < q; ++req) {
		ll t;
		cin >> t;
		ll k;
		cin >> k;
		ll ii = -1;
		if (t == 3) {
			cin >> ii;
			--ii;
		}
		reqs[req] = { k, t, ii, req };
	}
	ll sum_c = accumulate(c.begin(), c.end(), 0ll);
	sort(reqs.begin(), reqs.end());
	ll req_pos = 0;
	v<ll> req_res(q);
	ll lp = 0;
	for (ll iii = 0; iii < aa.size(); ++iii) {
		ll midiff = aa[iii];
		while (req_pos < q && get<0>(reqs[req_pos]) < lp + midiff) {
			auto [k, t, ii, i] = reqs[req_pos];
			if (t == 1) {
				req_res[i] = st.get_range(c.size() - 1, c.size()).p + 1;
			}
			else if (t == 2) {
				ll p = 0;
				tree_el t_e = st.get_range(0, c.size());
				ll cres = t_e.el + t_e.p * (k - lp);
				cres -= sum_c;
				req_res[i] = cres;
			}
			else {
				ll p = 0;
				tree_el te = st.get_range(invs[ii], invs[ii] + 1);
				ll cres = te.el + te.p * (k - lp);
				req_res[i] = cres;
			}
			++req_pos;
		}
		ll p = 0;
		//for (ll i = 0; i < n; ++i) {
			push_el p_e;
			p_e.deltael = 0;
			p_e.deltap = 0;
			p_e.deltael_p = midiff;
			st.mul_range(0, c.size(), p_e);
		//}
		for (ll x : a_c[iii]) {
			push_el p_e;
			p_e.deltael = 0;
			p_e.deltap = -1;
			p_e.deltael_p = 0;
			st.mul_range(x, c.size(), p_e);
		}
		lp += midiff;
	}
	while (req_pos < q) {
		auto [k, t, ii, i] = reqs[req_pos];
		if (t == 1) {
			req_res[i] = st.get_range(c.size() - 1, c.size()).p + 1;
		}
		else if (t == 2) {
			ll p = 0;
			tree_el t_e = st.get_range(0, c.size());
			ll cres = t_e.el + t_e.p * (k - lp);
			cres -= sum_c;
			req_res[i] = cres;
		}
		else {
			ll p = 0;
			tree_el te = st.get_range(invs[ii], invs[ii] + 1);
			ll cres = te.el + te.p * (k - lp);
			req_res[i] = cres;
		}
		++req_pos;
	}
	for (ll x : req_res) {
		cout << x << '\n';
	}
}