#include <iostream>
#include <string>
#include <algorithm>
#include <vector>
#include <map>

using namespace std;

using ll = long long;
template <typename T>
using v = vector<T>;

template <typename T>
istream& operator>> (istream& in, v<T>& a) {
	for (T& x : a) {
		in >> x;
	}
	return in;
}

struct tree_el {
	ll el, p;
	tree_el() : el(0), p(0) {};
	tree_el operator+(const tree_el& other) {
		tree_el res;
		res.el = el + other.el;
		res.p = p + other.p;
		return res;
	}
};

struct push_el {
	ll deltap;
	ll deltael_p;
	ll deltael;
	push_el() : deltap(0), deltael_p(0), deltael(0) {}
	tree_el operator*(const tree_el& other) {
		tree_el res;
		res.el = other.el + deltael + deltael_p * other.p;
		res.p = other.p + deltap;
		return res;
	}
};
push_el operator*(const push_el& a, const push_el& b) {
	push_el res;
	res.deltael = a.deltael + b.deltael + a.deltael_p * b.deltap;
	res.deltael_p = a.deltael_p + b.deltael_p;
	res.deltap = a.deltap + b.deltap;
	return res;
}

struct segment_tree {
	ll n;
	v<tree_el> st;
	v<push_el> p;
	tree_el neutral;
	push_el p_neutral;

	segment_tree(const v<tree_el>& a) : n(a.size()), st(4 * a.size()), neutral(), p_neutral() {
		p = v<push_el>(4 * a.size(), p_neutral);
		build(a, 0, 0, n);
	}

	tree_el combine_els(tree_el a, tree_el b) {
		return a + b;
	}

	tree_el build(const v<tree_el>& a, ll i, ll L, ll R) {
		if (R - L != 0) {
			if (R - L == 1) {
				return st[i] = a[L];
			}
			else {
				ll M = (L + R) >> 1;
				return st[i] = combine_els(build(a, (i << 1) + 1, L, M), build(a, (i << 1) + 2, M, R));
			}
		}
	}

	void push(ll i) {
		st[i] = p[i] * st[i];
		ll a = (i << 1) + 1;
		if (a < p.size()) {
			p[a] = p[i] * p[a];
			ll b = (i << 1) + 2;
			if (b < p.size()) {
				p[b] = p[i] * p[b];
			}
		}
		p[i] = p_neutral;
	}

	tree_el get_range(ll inL, ll inR, ll i = 0, ll L = 0, ll R = -1) {
		if (R == -1) {
			R = n;
		}
		if (R - L != 0) {
			push(i);
			if (R <= inL || L >= inR) {
				return neutral;
			}
			else if (L >= inL && R <= inR) {
				return st[i];
			}
			else {
				ll M = (L + R) >> 1;
				return combine_els(get_range(inL, inR, (i << 1) + 1, L, M), get_range(inL, inR, (i << 1) + 2, M, R));
			}
		}
	}

	tree_el mul_range(ll inL, ll inR, push_el inX, ll i = 0, ll L = 0, ll R = -1) {
		if (R == -1) {
			R = n;
		}
		if (R - L != 0) {
			push(i);
			if (R <= inL || L >= inR) {
				return st[i];
			}
			else if (L >= inL && R <= inR) {
				p[i] = inX * p[i];
				push(i);
				return st[i];
			}
			else {
				ll M = (L + R) >> 1;
				return st[i] = combine_els(mul_range(inL, inR, inX, (i << 1) + 1, L, M), mul_range(inL, inR, inX, (i << 1) + 2, M, R));
			}
		}
	}
};

int main() {
	ll n, q;
	cin >> n >> q;
	v<ll> c(n);
	cin >> c;
	v<pair<ll, ll>> cc(c.size());
	for (ll i = 0; i < c.size(); ++i) {
		cc[i] = { c[i], i };
	}
	sort(cc.begin(), cc.end(), greater<pair<ll, ll>>());
	v<pair<ll, v<pair<ll, ll>>>> r(1, { 0ll, cc });
	map<ll, v<ll>> a;
	for (ll i = 0; i < cc.size() - 1; ++i) {
		if (cc[i].first != cc[i + 1].first) {
			a[cc[i].first - cc[i + 1].first].push_back(i + 1);
		}
	}
	v<ll> ps(cc.size());
	{
		ll p = 0;
		for (ll i = 0; i < n; ) {
			ll j = i;
			while (j < n && cc[i].first == cc[j].first) {
				++j;
			}
			for (ll k = i; k < j; ++k) {
				ps[k] = p;
			}
			i = j;
			++p;
		}
	}
	a[0] = v<ll>();
	v<ll> aa(a.size() - 1);
	v<v<ll>> a_c(a.size() - 1);
	v<pair<ll, v<ll>>> a2(a.begin(), a.end());
	for (ll i = 0; i < (ll)a.size() - 1; ++i) {
		aa[i] = a2[i + 1].first - a2[i].first;
		a_c[i] = a2[i + 1].second;
	}
	v<ll> d;
	for (ll iii = 0; iii < aa.size(); ++iii) {
		ll midiff = aa[iii];
		ll p = 0;
		/*for (ll i = 0; i < n; ) {
			ll j = i;
			while (j < n && cc[i].first == cc[j].first) {
				++j;
			}
			for (ll k = i; k < j; ++k) {
				cc[k].first += p * midiff;
			}
			i = j;
			++p;
		}*/
		for (ll i = 0; i < n; ++i) {
			cc[i].first += ps[i] * midiff;
		}
		d.push_back(ps.back() + 1);
		for (ll x : a_c[iii]) {
			for (ll i = x; i < c.size(); ++i) {
				--ps[i];
			}
		}
		r.push_back({ r.back().first + midiff, cc });
	}
	d.push_back(1);
	for (ll req = 0; req < q; ++req) {
		ll t;
		cin >> t;
		ll k;
		cin >> k;
		ll ci = 0;
		while (ci < r.size() && r[ci].first <= k) {
			++ci;
		}
		--ci;
		if (t == 1) {
			cout << d[ci] << '\n';
		}
		else if (t == 2) {
			ll p = 0;
			ll cres = 0;
			for (ll i = 0; i < n; ) {
				ll j = i;
				while (j < n && r[ci].second[i].first == r[ci].second[j].first) {
					++j;
				}
				for (ll ck = i; ck < j; ++ck) {
					cres += r[ci].second[ck].first + p * (k - r[ci].first);
				}
				i = j;
				++p;
			}
			for (ll x : c) {
				cres -= x;
			}
			cout << cres << '\n';
		}
		else {
			ll ii;
			cin >> ii;
			--ii;

			ll p = 0;
			ll cres = 0;
			for (ll i = 0; i < n; ) {
				ll j = i;
				while (j < n && r[ci].second[i].first == r[ci].second[j].first) {
					++j;
				}
				for (ll ck = i; ck < j; ++ck) {
					if (r[ci].second[ck].second == ii) {
						cres = r[ci].second[ck].first + p * (k - r[ci].first);
					}
				}
				i = j;
				++p;
			}
			cout << cres << '\n';
		}
	}
}